#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""Tests for ticker() method"""

from pymarketcap import Pymarketcap
from pymarketcap.tests import type_test

pym = Pymarketcap()


def assert_types(res):
    map_types = {
        "name": str,
        "symbol": str,
        "added": str,
        "market_cap": (float, type(None)),
        "price": float,
        "circulating_supply": (float, type(None)),
        "volume_24h": (float, type(None)),
        "percent_change": (float, type(None)),
    }

    assert isinstance(res, list)
    assert len(res) > 0
    for currency in res:
        assert isinstance(currency, dict)
        for key, value in currency.items():
            type_test(map_types, key, value)


def test_types():
    res = pym.recently()
    assert_types(res)


def test_convert():
    res_btc = pym.recently(convert="BTC")

    # Test types
    assert_types(res_btc)

    # Test consistence
    res_usd = pym.recently()
    for curr_btc, curr_usd in zip(res_btc, res_usd):
        for key in curr_btc:
            assert type(curr_btc[key]) == type(curr_usd[key])



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from datetime import datetime, timedelta

import pytest

from pymarketcap import Pymarketcap
from pymarketcap.tests.utils import random_cryptocurrency
from pymarketcap.tests.historical import (
    assert_types,
    assert_consistence
)

pym = Pymarketcap()

def test_types():
    curr = random_cryptocurrency(pym)["website_slug"]
    print('(<currency>["website_slug"] == "%s")' % curr, end=" ")
    res = pym.historical(curr)
    assert_types(res)

def test_consistence():
    curr = random_cryptocurrency(pym)["website_slug"]
    print('(<currency>["website_slug"] == "%s")' % curr, end=" ")
    res = pym.historical(curr)
    assert_consistence(res)

def test_invalid():
    name = "BDAD)DAAS&/9324423OUVibb"
    with pytest.raises(ValueError) as excinfo:
        res = pym.historical(name)
    expected_msg = "Any cryptocurrency found matching name == %r." % name
    assert expected_msg in str(excinfo)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from re import findall as re_findall
from urllib.request import urlopen

from pymarketcap.tests.tokens import assert_types
from pymarketcap import Pymarketcap
pym = Pymarketcap()


def test_types():
    res = pym.tokens()
    assert_types(res)


def assert_number_of_tokens(res):
    req = urlopen("https://coinmarketcap.com/tokens/views/all/")
    data = req.read()
    req.close()
    indexes = int(
        re_findall(r'<td class="text-center">\n*(.+)\n*</td>', data.decode())[-1]
    )
    assert len(res) in [indexes-1, indexes, indexes+1]


def test_consistence():
    res = pym.tokens()
    assert_number_of_tokens(res)



# -*- coding: utf-8 -*-

import os
import sys
import asyncio
import pytest

sys.path.append(
    os.path.abspath(os.path.join(os.getcwd(), "pymarketcap"))
)

tests_cache_dir = os.path.join(os.path.abspath(os.path.dirname(__file__)), "cache")
if not os.path.exists(tests_cache_dir):
    os.mkdir(tests_cache_dir)

def pytest_addoption(parser):
    parser.addoption("--end2end", action="store_true",
                     default=False, help="Run slower tests.")

def pytest_collection_modifyitems(config, items):
    if not config.getoption("--end2end"):
        # Run end2end tests
        
        skip_end2end = pytest.mark.skip(reason="Need --end2end option to run")
        for item in items:
            if "end2end" in item.keywords:
                item.add_marker(skip_end2end)
    if sys.version_info < (3, 6,):
        skip_py36 = pytest.mark.skip(reason="Need Python v3.6 to run asynchronous tests.")
        for item in items:
            if "py36" in item.keywords:
                item.add_marker(skip_py36)

@pytest.yield_fixture()
def event_loop():
    loop = asyncio.get_event_loop()
    yield loop
    #loop.close()
    pass





#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import re
from json import loads

from precompiler.utils import GET

def get_badges_list():
    """Get list of convertible badges from coinmarketcap documentation"""
    url = "https://coinmarketcap.com/api/"
    res = GET(url).decode()
    #print(res)
    fiat_currs = re.search(r"Valid fiat currency values are:\s*(.*)\s*<br>", res)
    response = loads("[%s]" % fiat_currs.group(1))
    crypto_currs_raw = re.search(r"Valid cryptocurrency values are:\s*(.*)\s*</li>", res)
    crypto_currs = loads("[%s]" \
        % crypto_currs_raw.group(1).replace(" and", "").replace('" ', '", '))
    response.extend(crypto_currs)
    response.append("USD")
    return sorted(response)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import json
import os

import pytest

try:
    from pymarketcap import AsyncPymarketcap
except ImportError:
    pass
from pymarketcap.tests import disabled_decorator
from pymarketcap.tests.consts import asyncparms

asyncparms["debug"] = False

cache_file = os.path.join(
    os.path.abspath(os.path.dirname(os.path.dirname(__file__))),
    "cache", "every_currency.json"
)
exists_cache_file = os.path.exists(cache_file)

res = []

end2end_marked_if_not_cache = \
    pytest.mark.end2end if not exists_cache_file else disabled_decorator


@pytest.mark.py36
@pytest.mark.order1
@end2end_marked_if_not_cache
def test_all_currencies(event_loop):
    async def get_all_currencies():
        async with AsyncPymarketcap(**asyncparms) as apym:
            async for currency in apym.every_currency():
                res.append(currency)

    if exists_cache_file:
        with open(cache_file, "r") as f:
            for curr in json.loads(f.read()):
                res.append(curr)
    else:
        event_loop.run_until_complete(get_all_currencies())


def assert_any_field_in_response(field, readable_field):
    found = False
    for curr in res:
        value = curr[field]

        if isinstance(value, list):
            if len(value) > 0:
                found = True
        elif isinstance(value, str):
            if value != "":
                found = True
        elif isinstance(value, (int, float, bool)):
            found = True
        elif value is None:
            continue
        else:
            msg = "Case not anticipated. 'type(value) == %r'"
            raise NotImplementedError(msg % type(value))
    try:
        assert found == True
    except AssertionError as err:
        print("Any %s found searching all currencies." % readable_field \
              + "Check 'processer:currency()' function.")
        raise err

@pytest.mark.py36
@end2end_marked_if_not_cache
def test_chats():
    assert_any_field_in_response("chats", "chat")

@pytest.mark.py36
@end2end_marked_if_not_cache
def test_webs():
    assert_any_field_in_response("webs", "website")

@pytest.mark.py36
@end2end_marked_if_not_cache
def test_explorers():
    assert_any_field_in_response("explorers", "explorer")

@pytest.mark.py36
@end2end_marked_if_not_cache
def test_webs():
    assert_any_field_in_response("message_boards", "message board")

@pytest.mark.py36
@end2end_marked_if_not_cache
def test_source_code():
    assert_any_field_in_response("source_code", "source code link")

@pytest.mark.py36
@end2end_marked_if_not_cache
def test_announcement():
    assert_any_field_in_response("announcement", "announcement")

@pytest.mark.py36
@end2end_marked_if_not_cache
def test_mineable():
    assert_any_field_in_response("mineable", "'mineable == True' flag")

@pytest.mark.py36
@end2end_marked_if_not_cache
def test_rank():
    assert_any_field_in_response("rank", "'rank != None'")

@pytest.mark.py36
@end2end_marked_if_not_cache
def test_max_supply():
    assert_any_field_in_response("max_supply", "'max_supply != None'")

@pytest.mark.py36
@end2end_marked_if_not_cache
def test_total_supply():
    assert_any_field_in_response("total_supply", "'total_supply != None'")

@pytest.mark.py36
@end2end_marked_if_not_cache
def test_circulating_supply():
    assert_any_field_in_response("circulating_supply",
                                 "'circulating_supply != None'")



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from time import sleep
from urllib.request import urlopen

import pytest
from tqdm import tqdm

from pymarketcap import Pymarketcap
from pymarketcap.tests.utils import (
    random_cryptocurrency,
    random_exchange,
)

pym = Pymarketcap()

cryptocurrency = random_cryptocurrency(pym)
exchange = random_exchange(pym)["website_slug"]

base_public_api_url = "https://api.coinmarketcap.com/v2/"
base_web_url = "https://coinmarketcap.com/"
base_internal_api_url = "https://s2.coinmarketcap.com/"
base_graphs_api_url = "https://graphs2.coinmarketcap.com/"

endpoints = [
    ("_quick_search",           "%sgenerated/search/quick_search.json" % base_internal_api_url),
    ("_quick_search_exchanges", "%sgenerated/search/quick_search_exchanges.json" % base_internal_api_url),
    ("download_logo",           "%sstatic/img/coins/64x64/1.png" % base_internal_api_url),
    ("download_exchange_logo",  "%sstatic/img/exchanges/64x64/270.png" % base_internal_api_url),

    ("listings",                "%slistings/" % base_public_api_url),
    ("stats",                   "%sglobal/" % base_public_api_url),
    ("ticker",                 ["%sticker/" % base_public_api_url,
                                "%sticker/%s" % (base_public_api_url, cryptocurrency["id"])]),

    ("markets",                 "%scurrencies/%s/" % (base_web_url, cryptocurrency["website_slug"])),
    ("ranks",                   "%sgainers-losers/" % base_web_url),
    ("historical",              "%scurrencies/%s/historical-data/" % (base_web_url, cryptocurrency["website_slug"])),
    ("recently",                "%snew/" % base_web_url),
    ("exchange",                "%sexchanges/%s/" % (base_web_url, exchange)),
    ("exchanges",               "%sexchanges/volume/24-hour/all/" % base_web_url),
    ("tokens",                  "%stokens/views/all/" % base_web_url),

    ("graphs.currency",         "%scurrencies/%s/" % (base_graphs_api_url, cryptocurrency["website_slug"])),
    ("graphs.dominance",        "%sglobal/dominance/" % base_graphs_api_url),
    ("graphs.global_cap",      ["%sglobal/marketcap-total/" % base_graphs_api_url,
                                "%sglobal/marketcap-altcoin/" % base_graphs_api_url]),
]


def assert_up(ep):
    req = urlopen(ep)
    assert req.getcode() != 404
    req.close()


@pytest.mark.end2end
def test_endpoints():
    for i, (method, eps) in enumerate(tqdm(endpoints)):
        if i == 0:
            tqdm.write("tests/test_endpoints.py::test_endpoints")
        if not isinstance(eps, list):
            eps = [eps]
        for ep in eps:
            tqdm.write("%23s() --> %s" % (method, ep))
            assert_up(ep)
            sleep(.1)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import pytest

from pymarketcap.tests.utils import random_cryptocurrency
from pymarketcap.tests.currency import (
    assert_types,
    assert_consistence
)
from pymarketcap import Pymarketcap
pym = Pymarketcap()

def test_types():
    website_slug = random_cryptocurrency(pym)["website_slug"]
    print('(<currency>["website_slug"] == "%s")' % website_slug, end=" ")
    res = pym.currency(website_slug)
    assert_types(res)

def test_consistence():
    website_slug = random_cryptocurrency(pym)["website_slug"]
    print('(<currency>["website_slug"] == "%s")' % website_slug, end=" ")
    res = pym.currency(website_slug)
    assert_consistence(res)

def test_convert():
    website_slug = random_cryptocurrency(pym)["website_slug"]
    print('(<currency>["website_slug"] == "%s")' % website_slug, end=" ")
    res = pym.currency(website_slug, convert="BTC")

    assert_types(res)
    assert_consistence(res)

def test_invalid():
    name = "BDAD)DAAS&/9324423OUVibb"
    with pytest.raises(ValueError) as excinfo:
        res = pym.currency(name)
    expected_msg = "Any cryptocurrency found matching name == %r." % name
    assert expected_msg in str(excinfo)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from pymarketcap import Pymarketcap

pym = Pymarketcap()

currencies = pym.ranks()


def assert_any_field_in_response(field, readable_field):
    found = False
    for rank in ["gainers", "losers"]:
        for period in ["1h", "24h", "7d"]:
            for curr in currencies[rank][period]:
                value = curr[field]
                if isinstance(value, (int, float)):
                    found = True
                elif isinstance(value, str):
                    if value != "":
                        found = True
                elif value is None:
                    continue
                else:
                    msg = "Case not anticipated. 'type(value) == %r'"
                    raise NotImplementedError(msg % type(value))
    try:
        assert found == True
    except AssertionError as err:
        print("Any %s found searching all currencies." % readable_field \
              + "Check 'processer:ranks()' function.")
        raise err


def test_name():
    assert_any_field_in_response("name", "currency name")


def test_website_slug():
    assert_any_field_in_response("website_slug", "website_slug")


def test_symbol():
    assert_any_field_in_response("symbol", "currency symbol")


def test_percent_change():
    assert_any_field_in_response("percent_change", "percent_change")


def test_price():
    assert_any_field_in_response("price", "price")


def test_volume_24h():
    assert_any_field_in_response("volume_24h", "volume_24h")



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""``graphs()`` shared method test module."""

from datetime import datetime


def assert_types(res):
    assert isinstance(res, dict)
    for field, values in res.items():
        if field in ["website_slug", "symbol", "name"]:
            assert isinstance(values, str)
        elif field == "id":
            assert isinstance(values, int)
        else:
            assert isinstance(values, list)
            for tmp, value in values:
                assert isinstance(tmp, datetime)
                assert isinstance(value, (float, int))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from pymarketcap import Pymarketcap

pym = Pymarketcap()

currencies = pym.recently()


def assert_any_field_in_response(field, readable_field):
    found = False
    for curr in currencies:
        value = curr[field]
        if isinstance(value, (int, float)):
            found = True
        elif isinstance(value, str):
            if value != "":
                found = True
        elif value is None:
            continue
        else:
            msg = "Case not anticipated. 'type(value) == %r'"
            raise NotImplementedError(msg % type(value))
    try:
        assert found == True
    except AssertionError as err:
        print("Any %s found searching all currencies." % readable_field \
              + "Check 'processer:recently()' function.")
        raise err


def test_name():
    assert_any_field_in_response("name", "currency name")


def test_symbol():
    assert_any_field_in_response("symbol", "currency symbol")


def test_added():
    assert_any_field_in_response("added", "added date")


def test_circulating_supply():
    assert_any_field_in_response("circulating_supply", "circulating_supply")


def test_market_cap():
    assert_any_field_in_response("market_cap", "market_cap")


def test_percent_change():
    assert_any_field_in_response("percent_change", "percent_change")


def test_price():
    assert_any_field_in_response("price", "price")


def test_volume_24h():
    assert_any_field_in_response("volume_24h", "volume_24h")



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""Asynchronous Pymarketcap interface module."""

# Standard Python modules
import logging
from functools import partial
from json import loads
from datetime import datetime, date
from asyncio import (
    ensure_future,
    Queue,
)
from asyncio import TimeoutError as AsyncioTimeoutError

# External Python dependencies
from aiohttp import ClientSession
from tqdm import tqdm

# Internal Cython modules
from pymarketcap import processer, Pymarketcap

# Internal Python modules
from pymarketcap.consts import (
    DEFAULT_FORMATTER,
    DATETIME_MIN_TIME,
    DATETIME_MAX_TIME
)
from pymarketcap.util import cmc_timestamp

# Logging initialization
LOGGER_NAME = "/pymarketcap%s" % __file__.split("pymarketcap")[-1]
LOGGER = logging.getLogger(LOGGER_NAME)
LOGGER_HANDLER = logging.StreamHandler()
LOGGER_HANDLER.setFormatter(DEFAULT_FORMATTER)
LOGGER.addHandler(LOGGER_HANDLER)


class AsyncPymarketcap(ClientSession):
    """Asynchronous scraper for coinmarketcap.com

    Args:
        queue_size (int): Number of maximum simultanenous
           get requests performing together in methods
           involving several requests. As default ``10``.
        progress_bar (bool): Select ``True`` or ``False`` if you
            want to show a progress bar in methods that involve
            processing of several requests (requires :mod:`tqdm`
            module). As default, ``True``.
        consumers (int): Number of consumers resolving HTTP
            requests from an internal
            :class:`asyncio.Queue <~asyncio.Queue>`.
            As default, ``10``.
        timeout (int/float, optional): Limit max time
            waiting for a response. As default, ``15``.
        logger (logging.logger): As default is a logger
            with a :class:`~python.logging.StreamHandler`.
        debug (bool, optional): If ``True``, the logger
            level will be setted as :data:`~logging.DEBUG`.
            As default ``False``.
        sync (object, optional): Synchronous version instance
            of pymarketcap. As default
            :py:class:`pymarketcap.core.Pymarketcap`
        **kwargs: arguments that corresponds to the
            :class:`aiohttp.client.ClientSession <~aiohttp.ClientSession>`
            parent class.
    """
    def __init__(self, queue_size=10, progress_bar=True,
                 consumers=10, timeout=15, logger=LOGGER,
                 debug=False, sync=Pymarketcap(), **kwargs):
        super().__init__(**kwargs)
        self.timeout = timeout
        self.logger = logger
        self.sync = sync

        # Async queue
        self.queue_size = queue_size
        self.connector_limit = self.connector.limit
        self._responses = []
        self.progress_bar = progress_bar
        self.consumers = consumers

        self.graphs = type(
            "Graphs", (),
            {"every_currency": self._every_currency}
        )

        self.__start = None
        self.__end = None

        if debug:
            self.logger.setLevel(logging.DEBUG)

    # ====================================================================

                         #######   UTILS   #######

    async def _get(self, url):
        async with self.get(url, timeout=self.timeout) as response:
            return await response.text()

    async def _async_multiget(self, itr, build_url_callback,
                              num_of_consumers=None, desc=""):
        queue, dlq, responses = Queue(maxsize=self.queue_size), Queue(), []
        try:
            itr_len = len(itr)
        except TypeError:
            itr_len = 1000000

        if num_of_consumers is None:
            num_of_consumers = min(self.connector_limit, itr_len)

        consumers = [
            ensure_future(
                self._consumer(main_queue=queue, dlq=dlq, responses=responses)
            )
            for _ in range(num_of_consumers or self.connector_limit)
        ]
        dlq_consumers = [
            ensure_future(self._consumer(dlq, dlq, responses))
            for _ in range(num_of_consumers)
        ]
        await self._producer(itr, build_url_callback, queue, desc=desc)
        await queue.join()
        await dlq.join()

        all_consumers = consumers
        all_consumers.extend(dlq_consumers)
        for consumer in all_consumers:
            consumer.cancel()
        return responses

    async def _producer(self, items, build_url_callback, queue, desc=""):
        for item in tqdm(items, desc=desc, disable=not self.progress_bar):
            await queue.put(await build_url_callback(item))

    async def _consumer(self, main_queue, dlq, responses):
        while True:
            url = await main_queue.get()
            try:
                responses.append([url, await self._get(url)])
            except AsyncioTimeoutError:
                self.logger.debug("Problem with %s, Moving to DLQ" % url)
                await dlq.put(url)

            # Notify the queue that the item has been processed
            main_queue.task_done()

    # ====================================================================

                         #######   SCRAPER   #######

    async def _base_currency_url(self, curr):
        params = (self.sync.field_type(curr), curr)
        response = self.sync.cryptocurrency_by_field_value(*params)
        if not response:
            raise ValueError(
                "Any cryptocurrency found matching %s == %r." % params
            )
        else:
            curr = response["website_slug"]
        return "https://coinmarketcap.com/currencies/%s" % curr

    async def every_currency(self, currencies=None, convert="USD",
                             consumers=None):
        """Return general data from every currency in coinmarketcap
        passing a list of currencies as first parameter.
        As default returns data for all currencies.

        Args:
            currencies (list, optional): Iterator with all the
                currencies that you want to retrieve.
                As default ``None`` (:meth:`pymarketcap.Pymarketcap.coins`
                will be used in that case).
            convert (str, optional): Convert prices in response
                between "USD" and BTC. As default ``"USD"``.
            consumers (int, optional): Number of consumers
                processing the requests simultaneously.
                As default ``None``
                (see :attr:`pymarketcap.AsyncPymarketcap.consumers`).

        Returns (list): Data for all currencies.
        """
        if currencies is None:
            currencies = [
                curr["website_slug"]
                for curr in self.sync.cryptocurrencies
            ]

        res = await self._async_multiget(
            currencies,
            self._base_currency_url,
            consumers if consumers else self.consumers,
            desc="Retrieving every currency data "
                 "for %d currencies from coinmarketcap" % len(currencies)
        )
        for url, raw_res in res:
            self.logger.debug("Processing data from %s" % url)
            response = processer.currency(raw_res[20000:], convert.lower())
            response.update(
                self.sync.cryptocurrency_by_field_value(
                    "website_slug", url.split("/")[-1]
                )
            )
            yield response

    async def every_markets(self, currencies=None, convert="USD",
                            consumers=None):
        """Returns markets data from every currency in coinmarketcap
        passing a list of currencies as first parameter.
        As default returns data for all currencies.

        Args:
            currencies (list, optional): Iterator with
                all the currencies that you want to retrieve.
                As default ``None`` (:meth:`pymarketcap.Pymarketcap.coins`
                will be used in that case).
            convert (str, optional): Convert prices in
                response between "USD" and BTC.
                As default ``"USD"``.
            consumers (int, optional): Number of consumers
                processing the requests simultaneously.
                As default ``None``
                (see :attr:`pymarketcap.AsyncPymarketcap.consumers`).

        Returns (async iterator):
                Data for all currencies.
        """
        if currencies is None:
            currencies = [
                curr["website_slug"]
                for curr in self.sync.cryptocurrencies
            ]

        res = await self._async_multiget(
            currencies,
            self._base_currency_url,
            consumers if consumers else self.consumers,
            desc="Retrieving all markets "
                 "for %d currencies from coinmarketcap" % len(currencies)
        )
        for url, raw_res in res:
            self.logger.debug("Processing data from %s" % url)
            response = {
                "markets": processer.markets(raw_res[20000:], convert.lower()),
                "slug": url.split("/")[-1]
            }
            response.update(
                self.sync.cryptocurrency_by_field_value(
                    "website_slug", url.split("/")[-1]
                )
            )
            yield response

    async def _base_historical_url(self, curr):
        params = (self.sync.field_type(curr), curr)
        response = self.sync.cryptocurrency_by_field_value(*params)
        if not response:
            raise ValueError(
                "Any cryptocurrency found matching %s == %r." % params
            )
        else:
            curr = response["website_slug"]
        url = ("https://coinmarketcap.com/currencies"
               "/{currency}/historical-data"
               "?start={start:%Y%m%d}&end={end:%Y%m%d}")
        return url.format(currency=curr, start=self.__start, end=self.__end)

    async def every_historical(self, currencies=None,
                               start=datetime(2008, 8, 18),
                               end=datetime.now(),
                               revert=False,
                               consumers=None):
        """Returns historical data from every currency
        in coinmarketcap passing a list of currencies
        as first parameter. As default returns data
        for all currencies.

        Args:
            currencies (list, optional): Iterator with all
                the currencies that you want to retrieve.
                As default ``None`` (:meth:`pymarketcap.Pymarketcap.coins`
                will be used in that case).
            start (date, optional): Time to start scraping
                periods as ``datetime.datetime`` type.
                As default ``datetime(2008, 8, 18)``.
            end (date, optional): Time to end scraping periods
                as datetime.datetime type.
                As default ``datetime.now()``.
            revert (bool, optional): If ``False``, return first date
                first, in chronological order, otherwise returns
                reversed list of periods. As default ``False``.
            consumers (int, optional): Number of consumers
                processing the requests simultaneously.
                As default ``None``
                (see :attr:`pymarketcap.AsyncPymarketcap.consumers`).

        Returns (async iterator):
            Historical data for all currencies.
        """
        self.__start = start
        self.__end = end
        if currencies is None:
            currencies = [
                curr["website_slug"]
                for curr in self.sync.cryptocurrencies
            ]

        res = await self._async_multiget(
            currencies,
            self._base_historical_url,
            consumers if consumers else self.consumers,
            desc="Retrieving all historical data "
                 "for %d currencies from coinmarketcap" % len(currencies)
        )
        for url, raw_res in res:
            self.logger.debug("Processing data from %s" % url)
            response = {
                "history": processer.historical(
                    raw_res[50000:],
                    start,
                    end,
                    revert
                ),
                "slug": url.split("/historical-data")[0].split("/")[-2]
            }
            response.update(
                self.sync.cryptocurrency_by_field_value(
                    "website_slug", url.split("/")[-2]
                )
            )
            yield response

    async def _base_exchange_url(self, exc):
        return "https://coinmarketcap.com/exchanges/%s" % exc

    async def exchange(self, name, convert="USD"):
        convert = convert.lower()
        res = await self._get(await self._base_exchange_url(name))[20000:]
        return processer.exchange(res, convert)

    async def every_exchange(self, exchanges=None, convert="USD",
                             consumers=None):
        """Returns general data from every exchange
        in coinmarketcap passing a list of exchanges
        as first parameter. As default returns data
        for all exchanges.

        Args:
            exchanges (list, optional): Iterator with all
                the exchanges that you want to retrieve.
                As default ``None``
                (:meth:`pymarketcap.Pymarketcap.exchange_slugs`
                will be used in that case).
            convert (str, optional): Convert market_caps, prices,
                volumes and percent_changes between USD and BTC.
                As default ``"USD"``.
            consumers (int, optional): Number of consumers
                processing the requests simultaneously.
                As default ``None``
                (see :attr:`pymarketcap.AsyncPymarketcap.consumers`).

        Returns (async iterator):
            General data from all exchanges.
        """
        convert = convert.lower()

        if exchanges is None:
            exchanges = [
                exc["website_slug"]
                for exc in self.sync.cryptoexchanges
            ]

        res = await self._async_multiget(
            exchanges,
            self._base_exchange_url,
            consumers if consumers else self.consumers,
            desc="Retrieving all exchange data "
                 "for %d exchanges from coinmarketcap" % len(exchanges)
        )
        for url, raw_res in res:
            self.logger.debug("Processing data from %s" % url)
            response = processer.exchange(raw_res, convert)
            response.update(
                self.sync.exchange_by_field_value(
                    "website_slug", url.split("/")[-1]
                )
            )
            yield response

    # ====================================================================

                        #######   GRAPHS API   #######

    async def _base_graphs_currency_url(self, curr, start=None, end=None):
        params = (self.sync.field_type(curr), curr)
        response = self.sync.cryptocurrency_by_field_value(*params)
        if not response:
            raise ValueError(
                "Any cryptocurrency found matching %s == %r." % params
            )
        else:
            curr = response["website_slug"]

        url = "https://graphs2.coinmarketcap.com/currencies/%s" % curr

        if type(start) is date:
            start = datetime.combine(start, DATETIME_MIN_TIME)
        if type(end) is date:
            end = datetime.combine(end, DATETIME_MAX_TIME)

        if isinstance(start, datetime) and isinstance(end, datetime):
            start_tsmp = cmc_timestamp(start)
            end_tsmp = cmc_timestamp(end)
            url = "%s/%d/%d/" % (url.strip("/"), start_tsmp, end_tsmp)
        return url

    async def _every_currency(self, currencies=None, start=None, end=None,
                              use_auto_timeframe=False, consumers=None):
        """Returns graphs data from every currency in
        coinmarketcap passing a list of currencies as
        first parameter. As default returns graphs data
        for all currencies.

        Args:
            currencies (list, optional): Iterator with all
                the currencies that you want to retrieve.
                As default ``None`` (:meth:`pymarketcap.Pymarketcap.coins`
                will be used in that case).
            start (datetime, optional): Time to start retrieving
                graphs data in datetime. As default ``None``.
            end (datetime, optional): Time to end retrieving
                graphs data in datetime. As default ``None``.
            use_auto_timeframe (bool, optional): Use auto time frames same
                as fronted API. As default ``False``
            consumers (int, optional): Number of consumers
                processing the requests simultaneously.
                As default ``None``
                (see :attr:`pymarketcap.AsyncPymarketcap.consumers`).

        Returns (async iterator): Graphs data from all currencies.
        """
        url_kwargs = {}
        if use_auto_timeframe:
            url_kwargs.update(
                start=start,
                end=end,
            )

        if currencies is None:
            currencies = [
                curr["website_slug"]
                for curr in self.sync.cryptocurrencies
            ]

        res = await self._async_multiget(
            currencies,
            partial(self._base_graphs_currency_url, **url_kwargs),
            consumers if consumers else self.consumers,
            desc="Retrieving all graphs data for %d currencies "
                 "from coinmarketcap" % len(currencies)
        )
        for url, raw_res in res:
            self.logger.debug("Processing data from %s" % url)
            raw_res = loads(raw_res)
            response = processer.graphs(raw_res, start, end)
            response.update(
                self.sync.cryptocurrency_by_field_value(
                    "website_slug", url.split("/")[4]
                )
            )
            yield response



# -*- coding: utf-8 -*-

"""Shared interfaces tests modules."""
from time import sleep

from pymarketcap.errors import CoinmarketcapHTTPError


def type_test(map_types, key, value):
    """Test every type of the dictionary pairs
    key-values.

    Args:
        map_types (dict): Name of the keys and type
            or types that will be tested against it
            using ``isinstance(value, map_types[key])``
        key (str): Key that will be tested.
        value (str): Value name that will be tested.

    Raises:
       ``AssertionError`` if types and keys doesn't match.
    """
    _types = map_types.get(key)
    if _types is not None:
        msg = "Key: %s\nValue: %r\nTypes: %r\n" % (key, value, _types)
        try:
            assert isinstance(value, _types), msg
        except AssertionError as err:
            print(msg)
            raise err


def assert_cryptocurrency_or_exchange_types(res):
    for key, value in res.items():
        if key == "id":
            assert isinstance(value, int)
        else:
            assert isinstance(value, str)


def restart_if_http_error(res):
    def real_decorator(func):
        def wrapper(*args, **kwargs):
            ATTEMPTS = 10
            THROTTLE = 3  # Seconds between attempts
            error = True
            while error:
                try:
                    if ATTEMPTS > 0:
                        response = func()
                    else:
                        raise RuntimeError(
                            "Maximum number of attempts reached"
                        )
                except IndexError:
                    sleep(THROTTLE)
                except CoinmarketcapHTTPError:
                    sleep(THROTTLE)
                else:
                    error = False
                    ATTEMPTS -= 1
            return response
        return wrapper
    return real_decorator


def disabled_decorator(func):
    """This decorator disables the provided function, and does nothing."""
    def empty_func(*args,**kargs):
        pass
    return empty_func




"""API wraper and web scraper module."""

# Standard Python modules
import warnings
from re import (
    sub as re_sub,
    findall as re_findall
)
from time import time
from datetime import datetime, date
from json import loads
from urllib.request import urlretrieve
from urllib.error import HTTPError

# Internal Cython modules
from pymarketcap.consts import DATETIME_MIN_TIME, DATETIME_MAX_TIME
from pymarketcap.curl import get_to_memory
from pymarketcap import processer

# Internal Python modules
from pymarketcap.errors import (
    CoinmarketcapHTTPError,
    CoinmarketcapHTTPError404,
    CoinmarketcapTooManyRequestsError
)
from pymarketcap.util import cmc_timestamp

# HTTP errors mapper
http_errors_map = {
    "429": CoinmarketcapTooManyRequestsError,
    "404": CoinmarketcapHTTPError404,
}

http_error_numbers = [int(number) for number in http_errors_map.keys()]


cdef class Pymarketcap:
    """Synchronous class for retrieve data from https://coinmarketcap.com.

    Args:
        timeout (int, optional): Set timeout value for requests.
            As default ``20``.
        debug: (bool, optional): Show low level data in get requests.
            As default, ``False``.
        proxy_addr (bytes, optional): Proxy to use with Pymarketcap.
            As default, ``b""``.
    """
    cdef readonly dict _cryptocurrencies
    cdef readonly list _cryptoexchanges
    cdef readonly list _currencies_to_convert
    cdef readonly list _converter_cache

    cdef public long timeout
    cdef public object proxy_addr
    cdef public object graphs
    cdef public bint debug

    def __init__(self, timeout=15, debug=False, proxy_addr=b""):
        self.timeout = timeout
        self.debug = debug
        self.proxy_addr = proxy_addr

        #: object: Initialization of graphs internal interface
        self.graphs = type("Graphs", (), self._graphs_interface)

    # ====================================================================

                         #######   UTILS   #######

    @property
    def _graphs_interface(self):
        return {
            "currency": self._currency,
            "global_cap": self._global_cap,
            "dominance": self._dominance
        }

    cpdef _quick_search(self, exchanges=False):
        """Retrieve internal pymarketcap JSON cache
        files :file:`quick_search.json` and
        :file:`quick_search_exchanges.json` from
        ``https://s2.coinmarketcap.com/generated/search/``
        endpoint.

        Args:
            exchanges (bool, optional): If ``True``, returns
                :file:`quick_search_exchanges.json` and other file
                otherwise.

        Returns (list):
            All cryptocurrencies or exchanges information
        """
        cdef bytes url
        if exchanges:
            url = b"https://s2.coinmarketcap.com/generated/search/quick_search_exchanges.json"
        else:
            url = b"https://s2.coinmarketcap.com/generated/search/quick_search.json"
        return loads(self._get(url))

    @property
    def cryptocurrencies(self):
        """Return all cryptocurrencies listed at coinmarketcap.
        This is the cached version of public API listings method
        but without low level fields like ``"data"`` and ``"metadata"``.
        """
        if not self._cryptocurrencies:
            self._cryptocurrencies = self.listings()
        return self._cryptocurrencies["data"]

    cpdef cryptocurrency_by_field_value(self, unicode field, unicode value):
        """Returns a currency listed given a field and value
        if ``field`` and ``value`` parameters matches for a currency,
        otherwise returns ``None``.

        Args:
            field (str): Valid criptocurrency data field. Valid data fields
                are ``"id"``, ``"name"``, ``"symbol"`` and ``"website_slug"``.
            value (str): Value to compare for defined currency field.

        Returns (dict):
            A currency with ``"id"``, ``"name"``, ``"symbol"``
            and ``"website_slug"`` and their values mapped if the currency
            was found, else ``None``.
        """
        for currency in self.cryptocurrencies:
            if currency[field] == value:
                return currency
        return None

    @property
    def cryptoexchanges(self):
        """Returns all exchanges listed at coinmarketcap,
        as dictionaries with ``"name"``, ``"id"`` and
        ``"website_slug"`` keys.
        """
        if not self._cryptoexchanges:
            response = []
            for exchange in self._quick_search(exchanges=True):
                response.append({
                    "name": exchange["name"],
                    "id":   exchange["id"],
                    "website_slug": exchange["slug"]
                })
            self._cryptoexchanges = response
        else:
            response = self._cryptoexchanges
        return response

    cpdef exchange_by_field_value(self, unicode field, unicode value):
        """Returns a exchange listed given a field and value
        if ``field`` and ``value`` parameters matches for a exchange,
        otherwise returns ``None``.

        Args:
            field (str): Valid exchange data field. Valid data fields
                are ``"id"``, ``"name"`` and ``"website_slug"``.
            value (str): Value to compare for defined exchange field.

        Returns (dict):
            A currency with ``"id"``, ``"name"`` and ``"website_slug"``
            and their values mapped if the exchange was found,
            otherwise returns ``None``.
        """
        for exchange in self.cryptoexchanges:
            if exchange[field] == value:
                return exchange
        return None

    cpdef field_type(self, value):
        """Get a field type between ``"name"``, ``"symbol"``,
        ``"id"`` or ``"website_slug"``given a value. This
        method maybe is not strict, but used with previous
        methods works fine.

        Args:
            value (any): Value to guess field type.
        """
        try:
            _value = int(value)
        except ValueError:
            if value.isupper():
                return "symbol"
            elif value.islower():
                return "website_slug"
            else:
                return "name"
        else:
            return "id"

    @property
    def currencies_to_convert(self):
        res = self._currencies_to_convert
        if res:
            return res
        else:
            self._currencies_to_convert = self.__currencies_to_convert()
            return self._currencies_to_convert

    cpdef __currencies_to_convert(self):
        """Internal function for get currencies from and to convert
        values in ``convert()`` method. Don't use this, but cached
        :attr:`~pymarketcap.core.Pymarketcap.currencies_to_convert`
        instance attribute instead.

        Returns (list):
            All currencies that could be passed to
            :meth:`~pymarketcap.core.Pymarketcap.convert`.
        """
        res = self._get(b"https://coinmarketcap.com")
        response = re_findall(r'data-([a-z]+)="\d', res[-15000:-2000])
        response.append("USD")
        response.extend(
            [currency["symbol"] for currency in self.cryptocurrencies]
        )
        return sorted([curr.upper() for curr in response])

    @property
    def converter_cache(self):
        res = self._converter_cache
        if res:
            return res
        else:
            self._converter_cache = [self.currency_exchange_rates, time()]
            return self._converter_cache

    cpdef _get(self, char *url):
        """Internal function to make a HTTP GET request
        using the curl Cython bridge to C library or urllib
        standard library, depending of installation."""
        cdef int status
        req = get_to_memory(<char *>url, self.timeout,
                            <bint>self.debug, <char *>self.proxy_addr)
        status = req.status_code
        if status == 200:
            return req.text.decode()
        else:
            msg = "Status code -> %d | Url -> %s" % (status, url.decode())
            if status in http_error_numbers:
                raise http_errors_map[str(status)](msg)
            else:
                print("DEBUG: ")
                print(req.text)
                print(req.url)
                raise CoinmarketcapHTTPError(msg)

    # ====================================================================

                        #######   DEPRECATED   #######

    @property
    def coins(self):
        warnings.warn(
            "'coins()' method is deprecated. Use 'cryptocurrencies()' instead."
        )
        return [coin["website_slug"] for coin in self.cryptocurrencies]

    @property
    def symbols(self):
        warnings.warn(
            "'symbols()' method is deprecated. Use 'cryptocurrencies()' instead."
        )
        return [coin["symbol"] for coin in self.cryptocurrencies]

    @property
    def exchange_slugs(self):
        warnings.warn(
            "'exchange_slugs()' method is deprecated. Use 'exchanges()' instead."
        )
        return [exchange["website_slug"] for exchange in self.cryptoexchanges]

    @property
    def exchange_names(self):
        warnings.warn(
            "'exchange_names()' method is deprecated. Use 'exchanges()' instead."
        )
        return [exchange["name"] for exchange in self.cryptoexchanges]

    # ====================================================================

                        #######   PUBLIC API   #######

    cpdef listings(self):
        """List all criptocurrencies with their ids, names, symbol
        and website slug.

        .. seealso:

           `Coinmarketcap API documentation <https://coinmarketcap.com/api/>`__

        Returns (dict):
            Coinmarketcap API raw response.
        """
        return loads(self._get(b"https://api.coinmarketcap.com/v2/listings/"))

    cpdef stats(self, convert="USD"):
        """ Get global cryptocurrencies statistics.

        .. seealso:

           `Coinmarketcap API documentation <https://coinmarketcap.com/api/>`__

        Args:
            convert (str, optional): return 24h volume, and
                market cap in terms of another currency.
                See ``ticker_badges`` property to get valid values.
                As default ``"USD"``.

        Returns (dict):
            Global markets statistics on a raw response.
        """
        return loads(self._get(
            b"https://api.coinmarketcap.com/v2/global/?convert=%s" % convert.encode()
        ))

    @property
    def ticker_badges(self):
        """Badges in which you can convert prices in ``ticker()`` method."""
        return

    cpdef ticker(self, currency=None, limit=0, start=0, convert="USD"):
        """Get currencies with other aditional data.
        Only returns 100 currencies in each request. Use
        :py:meth:`pymarketcap.Pymarketcap.ticker_all` method
        for retrieve all currencies navegation through API
        pagination.


        .. seealso:

           `Coinmarketcap API documentation <https://coinmarketcap.com/api/>`__

        Args:
            currency (str, optional): Specify a currency to return data,
                that can be a ``name``, ``symbol``, ``id`` or
                ``website_slug`` fields from
                :py:class:`pymarketcap.core.Pymarketcap.cryptocurrencies`
                property. If you dont specify a currency, returns data
                for all in coinmarketcap. As default, ``None``.
            convert (str, optional): Allows to convert prices, 24h volumes
                and market capitalizations in terms of one of badges
                returned by ``ticker_badges`` property.
                As default, ``"USD"``.

        Returns (dict):
            Data from all currencies or a currency from coinmarketcap.
        """
        cdef short i, len_i
        if not currency:
            url =  "https://api.coinmarketcap.com/v2/ticker/?&convert=%s" % convert
            url += "&start=%d" % start
            url += "&limit=%s" % limit
            return loads(self._get(url.encode()))
        else:
            parms = (self.field_type(currency), currency)
            _id = self.cryptocurrency_by_field_value(*parms)["id"]
            url = "https://api.coinmarketcap.com/v2/ticker/%s" % _id
            url += "?convert=%s" % convert
            return loads(self._get(url.encode()))

    cpdef ticker_all(self, convert="USD"):
        """Get all currencies in coinmarketcap from
        :py:meth:`pymarketcap.Pymarketcap.ticker` method.
        Takes some time to be completed.

        Args:
            convert: (str, optional): Allows to convert prices, 24h volumes
                and market capitalizations in terms of one of badges
                returned by ``ticker_badges`` property.
                As default, ``"USD"``.

        Returns (list):
            All currencies metadata.
        """
        cdef short start = 1
        response = []

        num_cryptocurrencies = None
        while True:
            cryptocurrencies = self.ticker(start=start, convert="USD")
            if not num_cryptocurrencies:
                num_cryptocurrencies = \
                    cryptocurrencies["metadata"]["num_cryptocurrencies"]
            for curr in cryptocurrencies["data"].values():
                response.append(curr)
            start += 100
            if start > num_cryptocurrencies:
                break
        return response

    # ====================================================================

                       #######    WEB SCRAPER    #######

    @property
    def currency_exchange_rates(self):
        """Currency exchange rates against $ for all currencies (fiat + crypto).

        Returns (dict):
            All currencies rates used internally by coinmarketcap to calculate
            the prices shown.
        """
        cdef short start = 1

        res = self._get(b"https://coinmarketcap.com")
        rates = re_findall(
            r'data-([a-z]+)="(\d+\.*[\d|e|-]*)"', res[-10000:-2000]
        )
        response = {currency.upper(): float(rate) for currency, rate in rates}

        # Ticker API method pagination
        num_cryptocurrencies = None
        while True:
            cryptocurrencies = self.ticker(start=start)
            if not num_cryptocurrencies:
                num_cryptocurrencies = \
                    cryptocurrencies["metadata"]["num_cryptocurrencies"]
            for _id, currency in cryptocurrencies["data"].items():
                symbol = cryptocurrencies["data"][_id]["symbol"]
                usd_price = cryptocurrencies["data"][_id]["quotes"]["USD"]["price"]
                if usd_price:
                    response[symbol] = usd_price
            start += 100
            if start > num_cryptocurrencies:
                break
        return response

    cpdef convert(self, value, unicode currency_in, unicode currency_out):
        """Convert prices between currencies. Provide a value, the currency
        of the value and the currency to convert it and get the value in
        currency converted rate. For see all available currencies to convert
        see ``currencies_to_convert`` property.

        Args:
            value (int/float): Value to convert betweeen two currencies.
            currency_in (str): Currency in which is expressed the value passed.
            currency_out (str): Currency to convert.

        Returns (float):
            Value expressed in currency_out parameter provided.
        """
        if time() - self.converter_cache[1] > 600:
            self.converter_cache[0] = self.currency_exchange_rates
        try:
            if currency_in == "USD":
                return value / self.converter_cache[0][currency_out]
            elif currency_out == "USD":
                return value * self.converter_cache[0][currency_in]
            else:
                rates = self.converter_cache[0]
                return value * rates[currency_in] / rates[currency_out]
        except KeyError as err:
            msg = "Invalid currency: '%s'. " \
                + "See currencies_to_convert instance attribute."
            for param in [currency_in, currency_out]:
                if param not in self.currencies_to_convert:
                    raise ValueError(msg % param)
            raise err

    cpdef currency(self, unicode curr, convert="USD"):
        """Get currency metadata like total markets capitalization,
        websites, source code link, if mineable...

        Args:
            curr (str): Currency to get metadata. Can be a ``name``,
                ``symbol``, ``id`` or ``website_slug`` fields from
                :class:`pymarketcap.core.Pymarketcap.cryptocurrencies`.
            convert (str, optional): Currency to convert response
                fields ``total_markets_cap``, ``total_markets_volume_24h``
                and ``price`` between USD and BTC. As default ``"USD"``.

        Returns (dict):
            Aditional general metadata not supported by other methods.
        """
        parms = (self.field_type(curr), curr)
        response = self.cryptocurrency_by_field_value(*parms)
        if not response:
            raise ValueError(
                "Any cryptocurrency found matching %s == %r." % parms
            )
        else:
            curr = response["website_slug"]

        res = self._get(
            b"https://coinmarketcap.com/currencies/%s/" % curr.encode()
        )[20000:]

        response.update(processer.currency(res, convert.lower()))
        return response

    cpdef exchange(self, unicode exc, convert="USD"):
        """Obtain data from a exchange passed as argument.
        See :class:`pymarketcap.core.Pymarketcap.cryptoexchanges`
        property for obtain all posibles values.

        Args:
            exc (str): Exchange to retrieve data. Can be a ``name``,
                ``id`` or ``website_slug`` fields from
                :py:class:`pymarketcap.core.Pymarketcap.cryptoexchanges`.
            convert (str, optional): Convert prices and 24h volumes in
                return between USD and BTC. As default ``"USD"``.

        Returns (dict):
            Data from a exchange. Fields: ``"currency"``,
            ``"pair"``, ``"name"``, ``"volume_24h"`` (total),
            ``"price"``, ``"percent_volume"``, ``"updated"``.
            ``"slug"``, ``"website_slug"``, ``"id"``,
            ``"volume"``, ``"markets"``.
        """
        parms = (self.field_type(exc), exc)
        response = self.exchange_by_field_value(*parms)
        if not response:
            raise ValueError(
                "Any exchange found matching %s == %r." % parms
            )
        else:
            exc = response["website_slug"]

        res = self._get(
            b"https://coinmarketcap.com/exchanges/%s/" % exc.encode()
        )[20000:]

        response.update(processer.exchange(res, convert.lower()))
        return response

    cpdef exchanges(self, convert="USD"):
        """Get all exchanges in coinmarketcap ranked by volumes
        along with other metadata.

        Args:
            convert (str, optional): Convert volumes and prices
                between USD and BTC. As default ``"USD"``.

        Returns (list):
            Exchanges with markets and other data included.
        """
        cdef bytes url
        url = b"https://coinmarketcap.com/exchanges/volume/24-hour/all/"
        res = self._get(url)[45000:]
        return processer.exchanges(res, convert.lower())

    cpdef historical(self, unicode curr,
                     start=datetime(2008, 8, 18),
                     end=datetime.now(),
                     revert=False):
        """Get historical data for a currency.

        Args:
            curr (str): Currency to scrap historical data.
                Can be a ``name``,  ``"symbol"``, ``id``
                or ``website_slug`` fields from
                :py:class:`pymarketcap.core.Pymarketcap.cryptocurrencies`.
            start (date, optional): Time to start scraping
                periods as datetime.datetime type.
                As default :py:func:`datetime.datetime(2008, 8, 18)`.
            end (date, optional): Time to end scraping periods
                as datetime.datetime type. As default
                :py:func:`datetime.datetime.now()`.
            revert (bool, optional): If ``False``, return first date
                first, in chronological order, otherwise returns
                reversed list of periods. As default ``False``.

        Returns (list):
            Historical dayly OHLC for a currency.
        """
        parms = (self.field_type(curr), curr)
        response = self.cryptocurrency_by_field_value(*parms)
        if not response:
            raise ValueError(
                "Any cryptocurrency found matching %s == %r." % parms
            )
        else:
            curr = response["website_slug"]

        url = "https://coinmarketcap.com/currencies/%s/historical-data/" % curr
        _start = "%d" % start.year + "%02d" % start.month + "%02d" % start.day
        _end = "%d" % end.year + "%02d" % end.month + "%02d" % end.day
        url += "?start=%s" % _start + "&" + "end=%s" % _end

        res = self._get(url.encode())[50000:]
        response["history"] = processer.historical(res, start, end, revert)
        return response

    cpdef markets(self, unicode curr, convert="USD"):
        """Get available coinmarketcap markets data for a currency.

        Args:
            curr (str): Currency to get market data.
                Can be a ``name``,  ``"symbol"``, ``id``
                or ``website_slug`` fields from
                :py:class:`pymarketcap.core.Pymarketcap.cryptocurrencies`.
            convert (str, optional): Currency to convert response
                fields ``volume_24h`` and ``price`` between USD
                and BTC. As default ``"USD"``.

        Returns (list):
            Markets on wich provided currency is currently tradeable.
        """
        parms = (self.field_type(curr), curr)
        response = self.cryptocurrency_by_field_value(*parms)
        if not response:
            raise ValueError(
                "Any cryptocurrency found matching %s == %r." % parms
            )
        else:
            curr = response["website_slug"]

        res = self._get(
            b"https://coinmarketcap.com/currencies/%s/" % curr.encode()
        )[20000:]

        response["markets"] = processer.markets(res, convert.lower())
        return response

    cpdef ranks(self):
        """Returns gainers and losers for 1 hour, 24 hours and 7 days.

        Returns (dict):
            A dictionary with 2 keys (gainers and losers) whose values
            are the periods ``"1h"``, ``"24h"`` and ``"7d"``.
        """
        res = self._get(b"https://coinmarketcap.com/gainers-losers/")
        return processer.ranks(res)

    cpdef recently(self, convert="USD"):
        """Get recently added currencies along with other metadata.

        Args:
            convert (str, optional): Convert market_caps, prices,
                volumes and percent_changes between USD and BTC.
                As default ``"USD"``.

        Returns (list):
            Recently added currencies data.
        """
        res = self._get(b"https://coinmarketcap.com/new/")
        return list(processer.recently(res, convert.lower()))

    cpdef tokens(self, convert="USD"):
        """Get data from platforms tokens

        Args:
            convert (str, optional): Convert ``"market_cap"``,
                ``"price"`` and ``"volume_24h"`` values between
                USD and BTC. As default ``"USD"``.

        Returns (list):
            Platforms tokens data.
        """
        res = self._get(
            b"https://coinmarketcap.com/tokens/views/all/"
        )[40000:]
        return processer.tokens(res, convert.lower())

    # ====================================================================

                       #######   GRAPHS API   #######

    cpdef _currency(self, unicode curr, start=None, end=None,
                    use_auto_timeframe=False):
        """Get graphs data of a currency.

        Args:
            curr (str): Currency to retrieve graphs data.
            start (datetime, optional): Time to start retrieving
                graphs data in datetime type. As default ``None``.
            end (datetime, optional): Time to end retrieving
                graphs data in datetime type. As default ``None``.
            use_auto_timeframe (bool, optional): Use auto time frames same
                as fronted API. As default ``False``

        Returns (dict):
            Dict info with next keys: ``"market_cap_by_available_supply"``,
            ``"price_btc"``, ``"price_usd"``, ``"volume_usd":``
            and ``"price_platform"``.
            For each value, a list of lists where each one
            has two values [<datetime>, <value>]
        """
        parms = (self.field_type(curr), curr)
        response = self.cryptocurrency_by_field_value(*parms)
        if not response:
            raise ValueError(
                "Any cryptocurrency found matching %s == %r." % parms
            )
        else:
            curr = response["website_slug"]

        url = b"https://graphs2.coinmarketcap.com/currencies/%s/" % curr.encode()

        if use_auto_timeframe:
            if type(start) is date:
                start = datetime.combine(start, DATETIME_MIN_TIME)
            if type(end) is date:
                end = datetime.combine(end, DATETIME_MAX_TIME)

            if isinstance(start, datetime) and isinstance(end, datetime):
                start_tsmp = cmc_timestamp(start)
                end_tsmp = cmc_timestamp(end)
                url = b"%s/%d/%d/" % (url.strip(b"/"), start_tsmp, end_tsmp)

        res = loads(self._get(url))

        return processer.graphs(res, start, end)

    cpdef _dominance(self, start=None, end=None, use_auto_timeframe=False):
        """Get currencies dominance percentage graph

        Args:
            start (int, optional): Time to start retrieving
                graphs data in datetime. As default ``None``.
            end (optional, datetime): Time to start retrieving
                graphs data in datetime. As default ``None``.
            use_auto_timeframe (bool, optional): Use auto time frames same
                as fronted API. As default ``False``

        Returns (dict):
            Altcoins and dominance percentage values with timestamps.
        """
        url = b"https://graphs2.coinmarketcap.com/global/dominance/"
        if use_auto_timeframe:
            if type(start) is date:
                start = datetime.combine(start, DATETIME_MIN_TIME)
            if type(end) is date:
                end = datetime.combine(end, DATETIME_MAX_TIME)

            if isinstance(start, datetime) and isinstance(end, datetime):
                start_tsmp = cmc_timestamp(start)
                end_tsmp = cmc_timestamp(end)
                url = b"%s/%d/%d/" % (url.strip(b"/"), start_tsmp, end_tsmp)

        res = loads(self._get(url))
        return processer.graphs(res, start, end)

    cpdef _global_cap(self, bitcoin=True, start=None, end=None,
                      use_auto_timeframe=False):
        """Get global market capitalization graphs, including
        or excluding Bitcoin.

        Args:
            bitcoin (bool, optional): Indicates if Bitcoin will
                be includedin global market capitalization graph.
                As default ``True``.
            start (int, optional): Time to start retrieving
                graphs data in datetime. As default ``None``.
            end (optional, datetime): Time to start retrieving
                graphs data in datetime. As default ``None``.
            use_auto_timeframe (bool, optional): Use auto time frames same
                as fronted API. As default ``False``

        Returns (dict):
            Whose values are lists of lists with timestamp and values,
            a data structure with the keys: ``"volume_usd"`` and
            ``"market_cap_by_available_supply"``.
        """
        url = b"https://graphs2.coinmarketcap.com/global/marketcap-altcoin/"
        if bitcoin:
            url = b"https://graphs2.coinmarketcap.com/global/marketcap-total/"

        if use_auto_timeframe:
            if type(start) is date:
                start = datetime.combine(start, DATETIME_MIN_TIME)
            if type(end) is date:
                end = datetime.combine(end, DATETIME_MAX_TIME)

            if isinstance(start, datetime) and isinstance(end, datetime):
                start_tsmp = cmc_timestamp(start)
                end_tsmp = cmc_timestamp(end)
                url = b"%s/%d/%d/" % (url.strip(b"/"), start_tsmp, end_tsmp)

        res = loads(self._get(url))
        return processer.graphs(res, start, end)

    # ====================================================================

                       #######   INTERNAL API   #######

    cpdef download_logo(self, unicode curr, size=64, filename=None):
        """Download a currency image logo providing their size.

        Args:
            curr (str): Currency name, id, website_slug or symbol
                to download.
            size (int, optional): Size in pixels. Valid sizes are:
                ``[16, 32, 64, 128, 200]``. As default ``128``.
            filename (str, optional): Filename for store the logo.
                Doesn't include the extension (will be ".png").
                As default ``None``.

        Returns (str):
            Filename of downloaded file if all was correct.
        """
        parms = (self.field_type(curr), curr)
        response = self.cryptocurrency_by_field_value(*parms)
        if not response:
            raise ValueError(
                "Any cryptocurrency found matching %s == %r." % parms
            )
        else:
            _id = response["id"]

        url_schema = "https://s2.coinmarketcap.com/static/img/coins/%dx%d/%d.png"
        url = url_schema % (size, size, _id)
        if not filename:
            filename = "%s_%dx%d.png" % (response["website_slug"], size, size)
        try:
            res = urlretrieve(url, filename)
        except HTTPError as e:
            if e.code == 403:
                valid_sizes = [16, 32, 64, 128, 200]
                if size in valid_sizes:
                    raise ValueError(
                        ("Seems that %s currency doesn't allows to be " \
                         + "downloaded with size %dx%d. Try another size.") \
                             % (response["website_slug"], size, size)
                    )
                else:
                    raise ValueError("%dx%d is not a valid size." % (size, size))
            raise e
        else:
            return filename

    cpdef download_exchange_logo(self, unicode exc, size=64, filename=None):
        """Download a exchange logo passing his name or
        id or website slug as first parameter and
        optionally a filename without extension.

        Args:
            exc (str): Exchange name, id or website slug
                to download.
            size (int): Size in pixels. Valid values are:
                ``[16, 32, 64, 128, 200]``.
            filename (str, optional): Filename for store the logo,
                without include file extension (will be ".png").
                As default ``None``.

        Returns (str):
            Filename of downloaded file if all was correct.
        """
        parms = (self.field_type(exc), exc)
        response = self.exchange_by_field_value(*parms)
        if not response:
            raise ValueError(
                "Any exchange found matching %s == %r." % parms
            )
        else:
            _id = response["id"]
            _website_slug = response["website_slug"]

        filename = "%s_%dx%d.png" % (_website_slug, size, size) \
            if not filename else "%s.png" % filename

        url = "https://s2.coinmarketcap.com/static/img/exchanges/%dx%d/%s.png" % \
             (size, size, _id)

        try:
            res = urlretrieve(url, filename)
        except HTTPError as e:
            if e.code == 403:
                valid_sizes = [16, 32, 64, 128, 200]
                if size in valid_sizes:
                    raise ValueError(
                        ("Seems that %s exchange doesn't allows to be " \
                         + "downloaded with size %dx%d. Try another size.")  \
                            % (_website_slug, size, size)
                    )
                else:
                    raise ValueError("%dx%d is not a valid size." % (size, size))
            raise e
        else:
            return filename



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import json
import os

import pytest

try:
    from pymarketcap import AsyncPymarketcap
except ImportError:
    pass
from pymarketcap import Pymarketcap
from pymarketcap.tests.consts import asyncparms
from pymarketcap.tests.exchange import (assert_consistence, assert_types)

pym = Pymarketcap()

cache_file = os.path.join(
    os.path.abspath(
        os.path.dirname(os.path.dirname(os.path.dirname(__file__)))
    ),
    "cache", "every_exchange.json"
)

res = []

@pytest.mark.py36
@pytest.mark.end2end
def test_every_exchange(event_loop):
    async def wrapper():
        async with AsyncPymarketcap(**asyncparms) as apym:
            show_msg = True
            async for exc in apym.every_exchange():
                if show_msg:
                    print("Testing all responses...")
                    show_msg = False
                res.append(exc)

                assert_types(exc)
                assert_consistence(exc)
            assert isinstance(res, list)

        with open(cache_file, "w") as f:
            f.write(json.dumps(res, ensure_ascii=False))

    if os.path.exists(cache_file):
        with open(cache_file, "r") as f:
            for curr in json.loads(f.read()):
                res.append(curr)

        assert isinstance(res, list)
        for currency in res:
            assert_types(currency)
            assert_consistence(currency)
    else:
        event_loop.run_until_complete(wrapper())



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from re import findall
from urllib.request import urlopen

import pytest

from pymarketcap.tests import restart_if_http_error
from pymarketcap.tests.utils import random_exchange
from pymarketcap.tests.exchange import (
    assert_types,
    assert_consistence
)
from pymarketcap import Pymarketcap
pym = Pymarketcap()

@restart_if_http_error
def assert_number_of_markets(res):
    req = urlopen("https://coinmarketcap.com/exchanges/%s/" % res["website_slug"])
    data = req.read()
    req.close()
    indexes = findall(r'<td class="text-right">(\d+)</td>', data.decode())
    assert len(res["markets"]) == int(indexes[-1])

def test_types():
    exc = random_exchange(pym)["website_slug"]
    print('(<exchange>["website_slug"] == "%s")' % exc, end=" ")
    res = pym.exchange(exc)
    assert_types(res)

def test_consistence():
    exc = random_exchange(pym)["website_slug"]
    print('(<exchange>["website_slug"] == "%s")' % exc, end=" ")
    res = pym.exchange(exc)
    assert_consistence(res)
    assert_number_of_markets(res)

def test_convert():
    exc = random_exchange(pym)["website_slug"]
    print('(<exchange>["website_slug"] == "%s")' % exc, end=" ")
    res = pym.exchange(exc, convert="BTC")

    assert_types(res)
    assert_consistence(res)

def test_invalid():
    exc = "dabsfgbdsagubfeqbfeyfv"
    with pytest.raises(ValueError) as excinfo:
        pym.exchange(exc)
    expected_msg = "Any exchange found matching website_slug == %r" % exc
    assert expected_msg in str(excinfo.value)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""``historical()`` shared method test module."""

from datetime import datetime

from pymarketcap.tests import type_test

def assert_types(res):
    map_types = {
        "date":       datetime,
        "open":       float,
        "high":       float,
        "low":        float,
        "close":      (float, type(None)),
        "volume":     (float, type(None)),
        "market_cap": (float, type(None)),
        "name":       str
    }

    assert isinstance(res["history"], list)
    assert isinstance(res["symbol"], str)
    assert isinstance(res["website_slug"], str)
    assert isinstance(res["name"], str)
    assert isinstance(res["id"], int)
    for tick in res["history"]:
        assert isinstance(tick, dict)
        for key, value in tick.items():
            type_test(map_types, key, value)

def assert_consistence(res):
    for i, tick in enumerate(res["history"]):
        assert len(tick.keys()) == 7  # Check number of keys
        if len(res["history"]) > 1:
            if i < len(res["history"])-1:
                assert res["history"][i+1]["date"] < tick["date"]



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from random import randint
from datetime import datetime, timedelta
from pprint import pprint

from pymarketcap.tests.graphs import assert_types
from pymarketcap import Pymarketcap
pym = Pymarketcap()

def test_types():
    res = pym.graphs.dominance()

    assert_types(res)

def test_consistence():
    res = pym.graphs.dominance()

    fields = list(res.keys())
    assert "others" in fields

def test_start_end():
    today = datetime.now()
    days_ago = today - timedelta(days=randint(1, 15))

    res = pym.graphs.dominance(start=days_ago)
    key = list(res.keys())[0]
    possible_days_ago = [
        days_ago.day,
        (days_ago+timedelta(days=1)).day,
        (days_ago-timedelta(days=1)).day
    ]
    possible_days_today = [
        today.day,
        (today-timedelta(days=1)).day,
        (today+timedelta(days=1)).day
    ]
    assert res[key][-1][0].day in possible_days_today
    assert res[key][0][0].day in possible_days_ago



# -*- coding: utf-8 -*-

"""Pymarketcap utilities module."""

# Standard python modules
import sys
import time
from datetime import datetime


def get_timestamp(dt):
    if not isinstance(dt, datetime):
        raise ValueError('Argument `dt` must be is instance `datetime`.')

    if sys.version_info < (3, 3):
        return time.mktime(dt.timetuple()) + dt.microsecond / 1000000.
    return dt.timestamp()


def cmc_timestamp(dt):
    return int(get_timestamp(dt) * 1000)



# -*- coding: utf-8 -*-

from pymarketcap import Pymarketcap
pym = Pymarketcap()

ID = 1
STR_ID = "1"
FLOAT_ID = 1.1

def test_valid_id():
    assert "id" == pym.field_type(ID)

def test_str_id():
    assert "id" == pym.field_type(STR_ID)

def test_float_id():
    assert "id" == pym.field_type(FLOAT_ID)


NAME = "Bitcoin"
MALFORMED_NAME = "BitCoiN"

def test_valid_name():
    assert "name" == pym.field_type(NAME)

def test_malformed_name():
    assert "name" == pym.field_type(MALFORMED_NAME)


SYMBOL = "BTC"
SYMBOL_WITH_NUMBERS = "BTC2"

def test_valid_symbol():
    assert "symbol" == pym.field_type(SYMBOL)

def test_symbol_with_numbers():
    assert "symbol" == pym.field_type(SYMBOL_WITH_NUMBERS)


WEBSITE_SLUG = "bitcoin"

def test_valid_website_slug():
    assert "website_slug" == pym.field_type(WEBSITE_SLUG)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import json
import os

import pytest

try:
    from pymarketcap import AsyncPymarketcap
except ImportError:
    pass
from pymarketcap import Pymarketcap
from pymarketcap.tests.consts import asyncparms
from pymarketcap.tests.historical import (assert_consistence, assert_types)

pym = Pymarketcap()

cache_file = os.path.join(
    os.path.abspath(
        os.path.dirname(os.path.dirname(os.path.dirname(__file__)))
    ),
    "cache", "every_historical.json"
)

res = []

@pytest.mark.py36
@pytest.mark.end2end
def test_every_historical(event_loop):
    async def wrapper():
        async with AsyncPymarketcap(**asyncparms) as apym:
            show_msg = True
            async for (currency) in apym.every_historical():
                if show_msg:
                    print("Testing all responses...")
                    show_msg = False
                res.append(currency)

                assert_types(currency)
                assert_consistence(currency)
            assert isinstance(res, list)

    if os.path.exists(cache_file):
        with open(cache_file, "r") as f:
            for curr in json.loads(f.read()):
                res.append(curr)

        assert isinstance(res, list)
        for currency in res:
            assert_types(currency)
            assert_consistence(currency)
    else:
        event_loop.run_until_complete(wrapper())



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""``exchanges()`` shared method test module."""

from pymarketcap.tests import type_test

def assert_types(res):
    map_types = {
        "name":           str,
        "web":            str,
        "pair":           str,
        "volume":         (float, type(None)),
        "price":          float,
        "percent_volume": float
    }

    assert isinstance(res, list)
    for exc in res:
        assert isinstance(exc, dict)
        assert isinstance(exc["name"], str)
        assert isinstance(exc["markets"], list)
        for market in exc["markets"]:
            for key, value in market.items():
                type_test(map_types, key, value)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""Warning: use this example with caution,
it will use a lot of your memory."""

import asyncio
from pymarketcap import AsyncPymarketcap
from pprint import pprint
from datetime import datetime, timedelta
res = []

end = datetime.now().date()
start = end - timedelta(days=1)


async def main():
    async with AsyncPymarketcap(queue_size=50, consumers=50, debug=True) as apym:
        async for d in apym.graphs.every_currency(['bitcoin'], start=start, end=end, use_auto_timeframe=True):
            pprint(d)
            print(d.keys())

if __name__ == '__main__':
    loop = asyncio.get_event_loop()
    loop.run_until_complete(main())




"""This module process all core scraper methods raw responses."""

# Standard Python modules
import re
from datetime import datetime, date

from pymarketcap.consts import DATETIME_MIN_TIME, DATETIME_MAX_TIME
# RegEx parsing
PAIRS_REGEX = "[\s\$@\w\.]+/[\s\$@\w\.]+"

cpdef currency(res, convert):
    if convert == "usd":
        _total_markets_cap = re.search(
            r'data-currency-market-cap.+data-usd="(\?|\d+\.*\d*e*[-|+]*\d*)"', res
        )
        _total_markets_volume = re.search(
            r'data-currency-volume.+data-usd="(\?|\d+\.*\d*e*[-|+]*\d*)"', res
        )
        _price = re.search(r'quote_price.+data-usd="(\?|\d+\.*\d*e*[-|+]*\d*)"', res)
    else:  # convert == "btc"
        _total_markets_cap = re.search(
            r'data-format-market-cap.+data-format-value="(\?|\d+\.*\d*e*[-|+]*\d*)"', res
        )
        _total_markets_volume = re.search(
            r'data-format-volume-crypto.+data-format-value="(\?|\d+\.*\d*e*[-|+]*\d*)"', res
        )
        _price = re.search(
            r'data-format-price-crypto.+data-format-value="(\?|\d+\.*\d*e*[-|+]*\d*)"', res
        )

    vol_24h = _total_markets_volume.group(1)
    try:
        vol_24h = _total_markets_volume.group(1)
        vol_24h = float(vol_24h)
    except AttributeError:
        vol_24h = None
    except ValueError:
        if vol_24h != "?":
            raise
        vol_24h = None
    try:
        total_cap = _total_markets_cap.group(1)
        total_cap = float(total_cap)
    except AttributeError:
        total_cap = None
    except ValueError:
        if total_cap != "?":
            raise
        total_cap = None
    try:
        price = _price.group(1)
        price = float(price)
    except AttributeError:
        price = None
    except ValueError:
        if price != "?":
            raise
        price = None

    response = {
        "markets_cap": total_cap,
        "markets_volume_24h": vol_24h,
        "price": price
    }

    # Circulating, maximum and total supply
    supply = re.findall(
        r'data-format-supply.+data-format-value="(\?|\d+\.*\d*e*[-|+]*\d*)"', res
    )

    response["circulating_supply"] = float(supply[0]) if supply[0] != "?" else None
    if len(supply) > 1:
        response["max_supply"] = float(supply[-1])
    else:
        response["max_supply"] = None
    if len(supply) > 2:
        response["total_supply"] = float(supply[1])
    else:
        response["total_supply"] = None

    response["webs"] = re.findall(r'<a href="(.+)" target="_blank".*>Website\s*\d*</a>', res)

    response["explorers"] = re.findall(
        r'<a href="(.+)" target="_blank.*">Explorer\s*\d*</a>', res
    )

    source_code = re.search(r'<a href="(.+)" target="_blank".*>Source Code</a>', res)
    response["source_code"] = source_code.group(1) if source_code else None

    response["message_boards"] = re.findall(
        r'<a href="(.+)" target="_blank".*>Message Board\s*\d*</a>', res
    )

    response["chats"] = re.findall(
        r'<a href="(.+)" target="_blank".*>Chat\s*\d*</a>', res
    )

    response["mineable"] = True if re.search(r'label-warning">Mineable', res) else False

    try:
        response["rank"] = int(re.search(r'Rank (\d+)</span>', res).group(1))
    except AttributeError:
        response["rank"] = None

    announcement = re.search(r'<a href="(.+)" target="_blank".*>Announcement</a>', res)
    response["announcement"] = announcement.group(1) if announcement else None

    return response

cpdef markets(res, convert):
    sources = re.findall(r'<a.*?href="/exchanges/.+/".*?>([\s\w\.-]+)</a>', res)
    markets = re.findall(r'target="_blank">(%s)</a>' % PAIRS_REGEX, res)
    volume_24h = re.findall(r'ume" .*data-%s="(\d+\.\d+)' % convert, res)
    price = re.findall(r'"price" .*data-%s="(\?|\d+\.*\d*e*[-|+]*\d*)' % convert, res)
    perc_volume = re.findall(
        r'[^(]<span data-format-percentage data-format-value="(-*\d+\.*[\d|e|-]*[\d|e|-]*)">',
        res
    )
    updated = re.findall(r'text-right\s.*">(.+)</td>', res)

    return [
        {
            "source": src,
            "pair": mark,
            "volume_24h": float(vol),
            "price": float(price),
            "percent_volume": float(perc),
            "updated": up == "Recently"
        } for src, mark, vol, price, perc, up in zip(sources, markets, volume_24h,
                                              price, perc_volume, updated)
    ]

cpdef ranks(res):
    cdef int rank_len = 30

    names_slugs = re.findall(
        r'<a.*?href="/currencies/([^/]+?)/".*?>([^<>]+?)</a>', res
    )
    symbols = re.findall(r'<td class="text-left">(.+)</td>', res)
    volume_24h = re.findall(r'ume" .*data-usd="(\d+\.*[\d|e|-]*)"', res)
    price = re.findall(r'ice" .*data-usd="(\d+\.*[\d|e|-]*)"', res)
    percent_change = re.findall(r'right" .*data-usd="(-*\d+\.*[\d|e|-]*)"', res)

    periods = ["1h", "24h", "7d"]
    index_map ={
        "gainers": dict(zip(periods, (0, 30, 60))),
        "losers": dict(zip(periods, (90, 150, 120)))
    }

    return {
        rank: {
            period: [{
                "name": names_slugs[index_map[rank][period] + i][1].strip(),
                "website_slug": names_slugs[index_map[rank][period] + i][0],
                "symbol": symbols[index_map[rank][period] + i],
                "volume_24h": float(volume_24h[index_map[rank][period] + i]),
                "price": float(price[index_map[rank][period] + i]),
                "percent_change": float(percent_change[index_map[rank][period] + i])
            } for i in range(rank_len)]
            for period in periods
        }
        for rank in index_map
    }

cpdef historical(res, start, end, revert):
    cdef long len_i, i, i2, i3

    dates = re.findall(r'<td class="text-left">(.+)</td>', res)
    vol_marketcap = re.findall(r'cap data-format-value="(-|\d+\.*[\d+-e]*)"', res)
    ohlc = re.findall(r'fiat data-format-value="(-|\d+\.*[\d+-e]*)"', res)

    len_i = len(dates)
    i = 0
    i2 = min([len(vol_marketcap*2) - len(ohlc), 2])
    i3 = 0

    response = []
    for _ in range(len_i):
        date = datetime.strptime(dates[i], '%b %d, %Y')
        if date < start:
            continue
        else:
            if date <= end:
                try:
                    close = float(ohlc[i3+3])
                except ValueError:
                    close = None
                try:
                    volume = float(vol_marketcap[i2-1])
                except ValueError:
                    volume = None
                try:
                    market_cap = float(vol_marketcap[i2])
                except ValueError:
                    market_cap = None

                response.append({
                    "date": date,
                    "open": float(ohlc[i3]),
                    "high": float(ohlc[i3+1]),
                    "low": float(ohlc[i3+2]),
                    "close": close,
                    "volume": volume,
                    "market_cap": market_cap
                })
            else:
                break
        i += 1
        i2 += 2
        i3 += 4
    return list(reversed(response)) if revert else response

def recently(res, convert):
    names =  re.findall(
        r'<a.*?href="/currencies/[^/]+?/".*?>([^<>]+?)</a>', res
    )
    symbols = re.findall(r'<td class="text-left">(.+)</td>', res)
    added = re.findall(r'<td class="text-right.*">(Today|\d+ days ago)</td>', res)
    mcap = re.findall(r'cap .*data-%s="(\?|\d+\.*[\d|e|-|\+]*)"' % convert, res)
    prices = re.findall(r'price" .*data-%s="(\d+\.*[\d|e|-|\+]*)"' % convert, res)
    supply = re.findall(r'data-supply="(\?|\d+\.*[\d|e|-|\+]*)"', res)
    vol_24h = re.findall(r'ume" .*data-%s="(\?|\d+\.*[\d|e|-|\+]*)"' % convert, res)
    p_change = re.findall(r'data-symbol=".+" data-sort="(-*\d+\.\d*)"', res)

    for n, sym, add, mcp, pr, sup, vol, perc in zip(
            names, symbols, added, mcap, prices, supply, vol_24h, p_change
        ):
        try:
            perc_change = float(perc[0])
        except ValueError:
            perc_change = None
        try:
            market_cap = float(mcp)
        except ValueError:
            market_cap = None
        try:
            csupply = float(sup)
        except ValueError:
            csupply = None
        try:
            volume_24h = float(vol)
        except ValueError:
            volume_24h = None

        yield {
            "name": n,
            "symbol": sym,
            "added": add,
            "market_cap": market_cap,
            "price": float(pr),
            "circulating_supply": csupply,
            "volume_24h": volume_24h,
            "percent_change": perc_change
        }

cpdef exchange(res, convert):
    currencies = re.findall(r'".*?market-name.*?">(.+)</a>', res)
    pairs = re.findall(r'target="_blank">(%s)</a>' % PAIRS_REGEX, res)
    vol_24h = re.findall(r'ume" .*data-%s="(\?|\d+\.*\d*e{0,1}-{0,1}\d*)"' % convert, res)
    prices = re.findall(r'price" .*data-%s="(\d+\.*\d*e{0,1}-{0,1}\d*)"' % convert, res)
    perc_vols = re.findall(r'percentage data-format-value="(\d+\.*\d*e{0,1}-{0,1}\d*)"', res)
    updated = re.findall(r'text-right\s.*"\s*>(.+)</td>', res)

    twitter_username = re.search(r'target="_blank">(@.+)</a>', res)
    twitter_link = re.findall(r'"(https://twitter.com/[^\s]+)"', res) \
        if twitter_username else None

    formatted_name = re.search(r'class="logo-.+" alt="(.+)">', res).group(1)
    web = re.search(r'title="Website">.*href="\s*([^\s|"]+)', res)

    markets = []
    for curr, pair, vol, price, perc_vol, up in zip(
        currencies, pairs, vol_24h, prices, perc_vols, updated
        ):
        try:
            vol = float(vol)
        except ValueError:
            vol = None
        markets.append({
            "currency": curr,
            "pair": pair,
            "volume_24h": vol,
            "price": float(price),
            "percent_volume": float(perc_vol),
            "updated": up == "Recently"
        })
    if convert == "btc":
        try:
            total_volume = float(perc_vols[0])
        except (ValueError, IndexError):
            total_volume = None
    else:
        try:
            total_volume = float(
                re.search(r'currency-volume data-usd="(\d+\.*[\d|e|-|\+]*)">', res).group(1)
            )
        except (AttributeError, ValueError):
            total_volume = None

    return {
        "name": formatted_name,
        "web": web.group(1) if web else None,
        "volume": total_volume,
        "social": {
            "twitter": {
                "link": twitter_link[0] if twitter_link else None,
                "username": twitter_username.group(1) if twitter_username else None
            }
        },
        "markets": markets
    }

cpdef exchanges(res, convert):
    cdef int i
    exchanges =  re.findall(
        r'<a.*?href="/exchanges/([^/]+?)/">([^<>]+?)</a>', res
    )
    indexes = re.findall(r'<td>(\d+)</td>', res)
    currencies =  re.findall(
        r'<a.*?href="/currencies/[^/]+?/".*?>([^<>]+?)</a>', res
    )
    links_pairs = re.findall(r'<a href="(.*)" .*_blank">(%s)</a>' % PAIRS_REGEX, res)
    volumes = re.findall(
        r'class="text-right .*volume" .*data-%s="(\?|\d+\.*\d*e{0,1}-{0,1}\d*)"' % convert, res
    )
    prices = re.findall(r'ice" .*data-%s="(\?|\d+\.*\d*e{0,1}-{0,1}\d*)"' % convert, res)
    perc_volumes = re.findall(r'"percent-volume">(\d+\.*\d*)</span>', res)

    response = []
    for slug, exc in exchanges:
        markets = []
        for _ in indexes:
            i = int(_)
            try: vol = float(volumes[i-1])
            except ValueError: vol = None
            try: price = float(prices[i-1])
            except ValueError: price = None

            markets.append({
                "name": currencies[i-1],
                "web": links_pairs[i-1][0],
                "pair": links_pairs[i-1][1],
                "volume": vol,
                "price": price,
                "percent_volume": float(perc_volumes[i-1])
            })

            try:
                if indexes[i] == "1":
                    indexes = indexes[i:]
                    currencies = currencies[i:]
                    break
            except IndexError:
                break

        response.append({
            "name": exc,
            "website_slug": slug,
            "markets": markets,
        })

    return response

cpdef tokens(res, convert):
    names = re.findall(
        r'currency-name-container.*?".*?href="/currencies/[^/]+?/".*?>([^<>]+?)</a>',
        res
    )
    symbols = re.findall(
        r'currency-symbol.*?".*?href="/currencies/[^/]+?/".*?>([^<>]+?)</a>',
        res
    )
    platforms = re.findall(r'platform-name" data-sort="([^"]+)">', res)
    caps = re.findall(
        r'market-cap .*data-%s="(\?|\d+\.*\d*e{0,1}-{0,1}\d*)"' % convert, res
    )
    prices = re.findall(r'price" .*data-%s="(\?|\d+\.*\d*e{0,1}-{0,1}\d*)"' % convert, res)
    supplys = re.findall(r'data-supply="(None|\d+\.*\d*e{0,1}[+-]{0,1}\d*)"', res)
    vols_24h = re.findall(
        r'volume" .*data-%s="(None|\d+\.*\d*e{0,1}[+-]{0,1}\d*)"' % convert, res
    )

    response = []
    for n, sym, plat, mcap, price, sup, vol in zip(
        names, symbols, platforms, caps, prices, supplys, vols_24h
        ):
        if plat == "": plat = None
        try: mcap = float(mcap)
        except ValueError: mcap = None
        try: price = float(price)
        except ValueError: price = None
        try: sup = float(sup)
        except ValueError: sup = None
        try: vol = float(vol)
        except ValueError: vol = None
        response.append({
            "name": n,
            "symbol": sym,
            "platform": plat,
            "market_cap": mcap,
            "price": price,
            "circulating_supply": sup,
            "volume_24h": vol
        })
    return response

cpdef dict graphs(res, start, end):
    cdef dt_filter = _get_dt_filter(start, end)
    cdef str key
    cdef list value

    response = {}
    for key, value in res.items():
        group = []
        for _tmp, data in value:
            tmp = datetime.fromtimestamp(_tmp / 1000)
            if dt_filter(tmp):
                group.append([tmp, data])
        response[key] = group
    return response

cdef _get_dt_filter(start, end):
    if type(start) is date:
        start = datetime.combine(start, DATETIME_MIN_TIME)
    if type(end) is date:
        end = datetime.combine(end, DATETIME_MAX_TIME)

    cdef bint is_start = isinstance(start, datetime)
    cdef bint is_end = isinstance(end, datetime)
    cdef bint is_both = is_start and is_end

    if is_both:
        return lambda dt: start <= dt <= end
    elif is_start:
        return lambda dt: dt >= start
    elif is_end:
        return lambda dt: dt <= end
    else:
        return lambda dt: dt



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""Test wrapper properties"""

from pymarketcap import Pymarketcap
from pymarketcap.tests import assert_cryptocurrency_or_exchange_types
pym = Pymarketcap()


###   API METHOD CACHERS   ###

def test_cryptoexchanges():
    res = pym.cryptoexchanges

    # Test types
    assert isinstance(res, list)
    for exchange in res:
        assert_cryptocurrency_or_exchange_types(exchange)
        # Test consistency
        for field in ["website_slug", "id", "name"]:
            assert field in exchange
    assert len(res) > 0

def test_cryptocurrencies():
    res = pym.cryptocurrencies

    # Test types
    assert isinstance(res, list)
    for currency in res:
        assert_cryptocurrency_or_exchange_types(currency)
        for field in ["website_slug", "id", "name", "symbol"]:
            assert field in currency
    assert len(res) > 0


###   WEB SCRAPER METHOD CACHERS   ###

def test_currency_exchange_rates():
    # Test types
    res = pym.currency_exchange_rates
    for key, value in res.items():
        assert isinstance(key, str)
        assert isinstance(value, float)

def test_currencies_to_convert():
    # Test types
    currencies = pym.currencies_to_convert
    assert isinstance(currencies,  list)
    for curr in currencies:
        assert isinstance(curr, str)

    # Test consistence
    assert len(currencies) > 0



# -*- coding: utf-8 -*-

"""Constants used by Pymarketcap interfaces."""

import logging
from datetime import datetime

DEFAULT_FORMAT = "%(asctime)s %(levelname)-8s %(name)s:%(lineno)d: %(message)s"
DEFAULT_FORMATTER = logging.Formatter(DEFAULT_FORMAT)
DATETIME_MIN_TIME = datetime.min.time()
DATETIME_MAX_TIME = datetime.max.time()



# -*- coding: utf-8 -*-

from pymarketcap import Pymarketcap
from pymarketcap.tests import assert_cryptocurrency_or_exchange_types

pym = Pymarketcap()

SYMBOL = "BTC"
NAME = "Bitcoin"
WEBSITE_SLUG = "bitcoin"


def test_valid_cryptocurrency():
    res = pym.cryptocurrency_by_field_value("website_slug",
                                            WEBSITE_SLUG)
    assert res["name"] == NAME
    assert res["symbol"] == SYMBOL
    assert res["website_slug"] == WEBSITE_SLUG

    assert_cryptocurrency_or_exchange_types(res)


INVALID_WEBSITE_SLUG = "hnsdhkfyfd7Fye	ogufgk"


def test_invalid_cryptocurrency():
    res = pym.cryptocurrency_by_field_value("website_slug",
                                            INVALID_WEBSITE_SLUG)
    assert res is None



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
from datetime import datetime, timedelta

import pytest

try:
    from pymarketcap import AsyncPymarketcap
except ImportError:
    pass
from pymarketcap import Pymarketcap
from pymarketcap.tests.graphs import assert_types

pym = Pymarketcap()

end = datetime.now().date()
start = end - timedelta(days=1)

graphs_fields = (
    'market_cap_by_available_supply',
    'price_btc',
    'price_usd',
    'volume_usd',
)


async def graphs_currency(**kwargs):
    async with AsyncPymarketcap(debug=True,
                                queue_size=50,
                                consumers=50) as apym:
        currencies = []
        show_msg = True
        async for currency in apym.graphs.every_currency(**kwargs):
            if show_msg:
                print("Testing all responses...")
                show_msg = False
            currencies.append(currency)
        return currencies

@pytest.mark.py36
@pytest.mark.end2end
def test_every_graphs_currency(event_loop):
    async def wrapper():
        currencies = await graphs_currency()
        assert isinstance(currencies, list)

        for currency in currencies:
            assert_types(currency)

    event_loop.run_until_complete(wrapper())

@pytest.mark.py36
@pytest.mark.end2end
def test_every_graphs_currency_with_start(event_loop):
    async def wrapper():
        currencies = await graphs_currency(currencies=['bitcoin'], start=start)
        assert isinstance(currencies, list)
        for currency in currencies:
            assert_types(currency)

            for f in graphs_fields:
                assert f in currency

                values = currency[f]
                assert isinstance(values, list)
                assert len(values)

                dt = values[0][0]
                assert isinstance(dt, datetime)
                assert dt.date() == start

    event_loop.run_until_complete(wrapper())

@pytest.mark.py36
@pytest.mark.end2end
def test_every_graphs_currency_with_end(event_loop):
    async def wrapper():
        currencies = await graphs_currency(currencies=['bitcoin'], end=end)
        assert isinstance(currencies, list)
        for currency in currencies:
            assert_types(currency)

            for f in graphs_fields:
                assert f in currency

                values = currency[f]
                assert isinstance(values, list)
                assert len(values)

                dt = values[-1][0]
                assert isinstance(dt, datetime)
                assert dt.date() == end

    event_loop.run_until_complete(wrapper())

@pytest.mark.py36
@pytest.mark.end2end
def test_every_graphs_currency_with_start_end(event_loop):
    async def wrapper():
        currencies = await graphs_currency(
            currencies=['bitcoin'],
            start=start,
            end=end
        )
        assert isinstance(currencies, list)
        for currency in currencies:
            assert_types(currency)

            for f in graphs_fields:
                assert f in currency

                values = currency[f]
                assert isinstance(values, list)
                assert len(values)

                dt = values[0][0]
                assert isinstance(dt, datetime)
                assert dt.date() == start

                dt = values[-1][0]
                assert isinstance(dt, datetime)
                assert dt.date() == end

    event_loop.run_until_complete(wrapper())

@pytest.mark.py36
@pytest.mark.end2end
def test_every_graphs_currency_with_start_end_use_auto_timeframe(event_loop):
    async def wrapper():
        currencies = await graphs_currency(
            currencies=['bitcoin'],
            start=start,
            end=end,
            use_auto_timeframe=True,
        )
        assert isinstance(currencies, list)
        for currency in currencies:
            assert_types(currency)

            for f in graphs_fields:
                assert f in currency

                values = currency[f]
                assert isinstance(values, list)
                assert len(values) > 2

                start_dt = values[0][0]
                assert isinstance(start_dt, datetime)
                assert start_dt.date() == start

                end_dt = values[-1][0]
                assert isinstance(end_dt, datetime)
                assert end_dt.date() == end

                next_dt = values[1][0]
                assert isinstance(next_dt, datetime)
                assert next_dt - start_dt < timedelta(hours=1)

    event_loop.run_until_complete(wrapper())



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""Tests for ticker() method"""

from random import choice, randint

from pymarketcap.tests import type_test
from pymarketcap.tests.utils import random_cryptocurrency
from pymarketcap import Pymarketcap
pym = Pymarketcap()


def assert_types(res):
    map_types = {
        "id":                   int,
        "name":                 str,
        "symbol":               str,
        "rank":                 int,
        "quotes":               dict,
        "website_slug":         str,
        "circulating_supply":   (float, int, type(None)),
        "total_supply":         (float, int, type(None)),
        "max_supply":           (float, int, type(None)),
        "last_updated":         (int, type(None))
    }

    quotes_map_types = {
        "market_cap":           (float, int, type(None)),
        "percent_change_1h":    (float, type(None)),
        "percent_change_24h":   (float, type(None)),
        "percent_change_7d":    (float, type(None)),
        "price":                (float, type(None)),
        "volume_24h":           (float, type(None)),
    }

    assert isinstance(res, dict)
    for key, value in res.items():
        if key == "quotes":
            for _key, _value in value.items():
                for __key, __value in _value.items():
                    type_test(quotes_map_types, __key, __value)
        type_test(map_types, key, value)


def assert_consistence(res):
    assert len(res) == 10


def test_consistence():
    res = pym.ticker()
    for _id, curr_data in res["data"].items():
        assert_types(curr_data)
        assert_consistence(curr_data)

def test_limit():
    res = pym.ticker(limit=40)["data"]   # Max 100
    assert len(res) == 40
    res = pym.ticker(limit=90)["data"]
    assert len(res) == 90

def test_symbol():
    symbol = random_cryptocurrency(pym)["symbol"]
    print('(<currency>["symbol"] == "%s")' % symbol, end=" ")
    res = pym.ticker(symbol)

    if res:
        assert_types(res["data"])
        assert_consistence(res["data"])

def test_name():
    name = random_cryptocurrency(pym)["name"]
    print('(<currency>["name"] == "%s")' % name, end=" ")
    res = pym.ticker(name)

    if res:
        assert_types(res["data"])
        assert_consistence(res["data"])

def test_start():
    pos = randint(0, len(pym.cryptocurrencies))
    res = pym.ticker(start=pos)

    if res:
        curr_id = list(res["data"].keys())[0]
        assert res["data"][curr_id]["rank"] - pos <= 50
        assert_types(res["data"][curr_id])
        assert_consistence(res["data"][curr_id])

def test_convert():
    for currency in [True, False]:  # With and without currency
        symbol = None
        badge = choice(pym.ticker_badges)
        if currency:
            symbol = random_cryptocurrency(pym)["symbol"]
            print('(<currency>["name"] == "%s") - Badge: %s)' \
                      % (symbol, badge), end=" | ")
        else:
            print("(Badge: %s)" % badge, end=" ")

        res = pym.ticker(currency=symbol, convert=badge)
        if currency and res:
            assert_types(res["data"])
            assert_consistence(res["data"])


def test_ticker_all():
    res = pym.ticker_all()
    if res:
        for curr in res:
            assert_types(curr)
            assert_consistence(curr)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""``tokens()`` synchronous method test module."""

from pymarketcap.tests import type_test

def assert_types(data):
    map_types = {
        "name":               str,
        "symbol":             str,
        "platform":           (str, type(None)),
        "market_cap":         (float, type(None)),
        "price":              (float, type(None)),
        "circulating_supply": (float, type(None)),
        "volume_24h":         (float, type(None))
    }
    for currency in data:
        for key, value in currency.items():
            type_test(map_types, key, value)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""``currency()`` shared method test module."""

from pymarketcap.tests import type_test


def assert_types(res):
    map_types = {
        "symbol":             str,
        "website_slug":       str,
        "source_code":        (str, type(None)),
        "announcement":       (str, type(None)),
        "explorers":          list,
        "markets_volume_24h": (float, type(None)),
        "price":              (float, type(None)),
        "rank":               (int, type(None)),
        "markets_cap":        (float, type(None)),
        "chats":              list,
        "message_boards":     list,
        "circulating_supply": (float, type(None)),
        "total_supply":       (float, type(None)),
        "max_supply":         (float, type(None)),
        "mineable":           bool,
        "webs":               list,
        "name":               str,
        "id":                 int
    }
    assert isinstance(res, dict)
    for key, value in res.items():
        type_test(map_types, key, value)


def assert_consistence(res):
    assert len(list(res.keys())) == 18



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from pymarketcap import Pymarketcap

pym = Pymarketcap()

exchanges = pym.exchanges()


def assert_any_field_in_response(field, readable_field):
    found = False
    for exc in exchanges:
        value = exc[field]

        if isinstance(value, list):
            found = True
            for market in value:
                for _key, _value in market.items():
                    if _value is None:
                        found = False
                        break
                if found == False:
                    break
        elif isinstance(value, str):
            if value != "":
                found = True
        elif value is None:
            continue
        else:
            msg = "Case not anticipated. 'type(value) == %r'"
            raise NotImplementedError(msg % type(value))
    try:
        assert found == True
    except AssertionError as err:
        print("Any %s found searching all currencies." % readable_field \
              + "Check 'processer:exchanges()' function.")
        raise err


def test_name():
    assert_any_field_in_response("name", "exchange name")


def test_markets():
    assert_any_field_in_response("markets", "market")



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from re import findall as re_findall
from urllib.request import urlopen

from pymarketcap.tests import restart_if_http_error
from pymarketcap.tests.exchanges import assert_types
from pymarketcap import Pymarketcap
pym = Pymarketcap()


@restart_if_http_error
def assert_number_of_exchanges(res):
    req = urlopen("https://coinmarketcap.com/exchanges/volume/24-hour/all/")
    data = req.read()
    req.close()
    indexes = re_findall(r'"volume-header">(\d+)\.', data.decode())
    assert len(res) == int(indexes[-1])

def assert_consistence(res):
    assert len(res) > 0
    assert_number_of_exchanges(res)
    for exc in res:
        for market in exc["markets"]:
            assert market["pair"].count("/") == 1

def test_types():
    assert_types(pym.exchanges())

def test_consistence():
    res = pym.exchanges()
    assert_consistence(res)

def test_convert():
    assert_types(pym.exchanges(convert="BTC"))
    assert_consistence(pym.exchanges(convert="BTC"))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import pytest

try:
    from pymarketcap import AsyncPymarketcap
except ImportError:
    pass
from pymarketcap import Pymarketcap
from pymarketcap.tests.consts import asyncparms

@pytest.mark.py36
def test_sync_interface(event_loop):
    async def wrapper():
        async with AsyncPymarketcap(**asyncparms) as apym:
            sync_interface = apym.sync
        assert isinstance(sync_interface, Pymarketcap)

    event_loop.run_until_complete(wrapper())



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from datetime import datetime, timedelta
from random import randint

from pymarketcap import Pymarketcap
from pymarketcap.tests.graphs import assert_types
from pymarketcap.tests.utils import random_cryptocurrency

pym = Pymarketcap()


def test_types():
    curr = random_cryptocurrency(pym)["website_slug"]
    res = pym.graphs.currency(curr)
    print('(<currency>["website_slug"] == "%s")' % curr, end=" ")

    assert_types(res)


def test_consistence():
    curr = random_cryptocurrency(pym)["website_slug"]
    res = pym.graphs.currency(curr)
    print('(<currency>["website_slug"] == "%s")' % curr, end=" ")

    assert len(res) in [4, 5]

    fields = [
        "market_cap_by_available_supply",
        "price_btc",
        "price_usd",
        "volume_usd",
        "price_platform",
    ]

    for field, values in res.items():
        assert field in fields


def test_start_end():
    days_ago = timedelta(days=randint(1, 15))
    start = datetime.now() - days_ago
    end = datetime.now()
    curr = random_cryptocurrency(pym)["website_slug"]
    print('(<currency>["website_slug"] == %s | Start: %r | End: %r)' % (
    curr, start, end),
          end=" ")
    for startend in [True, False]:
        params = {"start": start, "end": end}
        if not startend:
            params["start"], params["end"] = (None, None)
        res = pym.graphs.currency(curr, **params)

        # Test types
        assert isinstance(res, dict)

        for key, _list in res.items():
            assert isinstance(_list, list)
            for elem, value in _list:
                assert isinstance(elem, datetime)
                assert type(value) in [int, float]

        # Test consistence
        if end.day > 2:
            assert end.day >= res["price_btc"][-1][0].day
        if startend:
            first_day = res["price_usd"][0][0]
            next_day = res["price_usd"][0][0] + timedelta(days=1)
            assert start <= next_day



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import os
from time import sleep
from random import choice

import pytest
from tqdm import tqdm

from pymarketcap import Pymarketcap
pym = Pymarketcap()

exchange_names = pym.exchange_names
exchange_slugs = pym.exchange_slugs

def assert_consistence(res):
    assert len(res) > 0
    assert isinstance(res, str)
    assert os.path.exists(res)
    os.remove(res)
    assert os.path.exists(res) == False

def test_consistence():
    print("")
    for size in tqdm([16, 32, 64, 128, 200],
                     desc="Testing exchange logo downloads for all sizes"):
        attempts = 20
        while attempts > 0:
            _assert = True
            exc = choice(exchange_slugs)
            tqdm.write("(Exchange: %s | Size: %d)" % (exc, size))
            try:
                res = pym.download_exchange_logo(exc, size=size)
            except ValueError as e:
                print(e)
                attempts -= 1
                _assert = False
                if attempts == 0:
                    break
            else:
                break
        if _assert:
            assert res == "%s_%dx%d.png" % (exc, size, size)
            sleep(.5)

            assert os.path.exists(res)
            os.remove(res)
            assert os.path.exists(res) == False
        if attempts == 0:
            raise AssertionError("0 exchange logos downloaded. Check it!")

def test_invalid():
    # Invalid exchange
    exc = "PIDBASDVBHB934VHJ"

    with pytest.raises(ValueError) as excinfo:
        pym.download_exchange_logo(exc)
    assert "Exchange %s not found." % exc in str(excinfo)

    # Invalid size
    exc = choice(exchange_names)
    size = 47365
    print("(Exchange: %s)" % exc, end=" ")
    with pytest.raises(ValueError) as excinfo:
        pym.download_exchange_logo(exc, size=size)
    assert "%dx%d is not a valid size." % (size, size) in str(excinfo)

def test_download_passing_name():
    exc = choice(exchange_names)
    print("(Exchange: %s)" % exc, end=" ")
    res = pym.download_exchange_logo(exc)
    assert_consistence(res)

def test_download_passing_slug():
    exc = choice(exchange_slugs)
    print("(Exchange: %s)" % exc, end=" ")
    res = pym.download_exchange_logo(exc)
    assert_consistence(res)




#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from pymarketcap.tests.ranks import assert_types
from pymarketcap import Pymarketcap
pym = Pymarketcap()

def test_types():
    res = pym.ranks()
    assert_types(res)

def test_consistence():
    res = pym.ranks()

    # Check if a currency is repeated
    for rank, rdata in res.items():
        assert rank in ["gainers", "losers"]
        for period, pdata in rdata.items():
            assert period in ["1h", "24h", "7d"]




#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from pymarketcap import Pymarketcap

def main():
    cmc = Pymarketcap()

    # Get all exchanges ranked by volumes
    excs = cmc.exchanges()
    print(excs[0])  # Show first exchange

if __name__ == "__main__":
    main()




#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import os
import sys
from shlex import split as parse
from subprocess import check_output

from setuptools import find_packages, setup
from setuptools.extension import Extension

CURR_DIR = os.path.dirname(__file__)
REQ_PATH = os.path.join(CURR_DIR, "requirements.txt")


def simple_call(command):
    try:
        return check_output(parse(command))
    except Exception as e:
        return 1


def spec_sys_calls(command):
    if sys.platform.startswith('linux') or sys.platform.startswith('darwin'):
        command = [command, "sudo %s" % command]
    return [command]


def install_cython_via_pip():
    code = 1
    calls = spec_sys_calls("pip3 install Cython")[0]
    for i, command in enumerate(calls):
        code = simple_call(command)
        print(code)
        if i >= len(calls) or code == 0:
            break
    return code


try:
    from Cython.Build import cythonize
except ImportError:  # Cython required
    exit("Cython not found. You need to install Cython before pymarketcap.")

code = 1
base_call = "pip3 install -r %s" % REQ_PATH
calls = spec_sys_calls(base_call)
for i, command in enumerate(calls):
    code = simple_call(command)
    if i >= len(calls) or code == 0:
        break

if code == 0:
    print("Dependencies installed successfully.")
else:
    print("Error installing dependencies, installing pymarketcap anywhere...")

# ===========  Precompiler  ===========
if "sdist" not in sys.argv:
    from precompiler import run_builder, run_unbuilder

    source = os.path.join(os.getcwd(), "pymarketcap", "core.pyx")
    built = run_builder(source)
    if not built:
        if run_unbuilder():
            if run_builder(source):
                pass
            else:
                exit("Error building pymarketcap.")
        else:
            exit("Error building pymarketcap.")

# Check if minimum Python3.6
PYTHON_VERSION = sys.version_info
if PYTHON_VERSION < (3, 6,):
    if PYTHON_VERSION > (3, 5,):
        if "--quiet" not in sys.argv:
            while True:
                msg_scheme = "\nPython version:\n%s\n\nYou need " \
                           + "to install almost Python3.6 in order to use " \
                           + "the asynchronous Pymarketcap interface.\n" \
                           + "Install pymarketcap without it anyway? Y/N: "
                msg = msg_scheme % str(PYTHON_VERSION)
                cont = str(input(msg)).lower()
                if cont == "n":
                    exit("Installation cancelled.")
                elif cont != "y":
                    print("Invalid option.")
                else:
                    print("Installing pymarketcap...")
                    break
    else:
        exit("You need almost Python3.5 version to install pymarketcap.")

# ===========  Cython compilation  ===========
COMPILE_CURL = True

# Windows
if sys.platform.startswith('win32') and "--force-curl" not in sys.argv:
    COMPILE_CURL = False

if "--force-curl" in sys.argv:
    sys.argv.remove("--no-curl")

# Building on ReadTheDocs (very crazy):
if "--no-curl" in sys.argv or os.environ.get("READTHEDOCS") == "True":
    COMPILE_CURL = False
    if "--no-curl" in sys.argv:
        sys.argv.remove("--no-curl")


def declare_cython_extension(ext_name, libraries=None):
    """Declare a Cython extension module for setuptools.

    Args:
        ext_name (str):
            Absolute module name, e.g. use `mylibrary.mypackage.mymodule`
            for the Cython source file `mylibrary/mypackage/mymodule.pyx`.
        libraries (list):
            Libraries needed to build the extension.

    Returns (object):
        Extension object that can be passed to ``setuptools.setup``.
    """
    ext_path = ext_name.replace(".", os.path.sep) + ".pyx"

    return Extension(ext_name, [ext_path], libraries=libraries)


ext_modules = [
    declare_cython_extension("pymarketcap.core"),
    declare_cython_extension("pymarketcap.processer"),
]

package_data = {"pymarketcap": ["core.pyx", "processer.pyx", ]}

if COMPILE_CURL:
    ext_modules.append(
        declare_cython_extension("pymarketcap.curl", libraries=["curl"])
    )
    package_data["pymarketcap"].extend(["curl.pyx", "curl.pxd"])
else:
    core_path = os.path.join(CURR_DIR, "pymarketcap", "core.pyx")
    with open(core_path, "r") as f:
        content = f.readlines()

    for i, line in enumerate(content):
        if "from pymarketcap.curl import get_to_memory" in line:
            content[i] = line.replace("curl", "url")
            break

    with open(core_path, "w") as f:
        f.writelines(content)

ext_modules = cythonize(ext_modules)

# ===========  Package metadata  ===========
with open(os.path.join(CURR_DIR, 'README.rst')) as f:
    LONG_DESC = f.read()

with open(REQ_PATH, "r") as f:
    REQ = [line.strip("\n") for line in f.readlines()]

author, author_email = ("Álvaro Mondéjar Rubio", "mondejar1994@gmail.com")

install = setup(
    name="pymarketcap",
    version="4.0.0013",
    url="https://github.com/mondeja/pymarketcap",
    download_url="https://github.com/mondeja/pymarketcap/archive/master.zip",
    author=author,
    maintainer=author,
    author_email=author_email,
    maintainer_email=author_email,
    platforms=['any'],
    license="BSD License",
    description="Python3 API and web scraper for coinmarketcap.com.",
    long_description=LONG_DESC,
    keywords=["coinmarketcap", "cryptocurrency", "API", "wrapper",
              "scraper", "json", "BTC", "Bitcoin", "C", "Curl", "Cython"],
    install_requires=REQ,
    packages=find_packages(exclude=["precompiler"]),
    ext_modules=ext_modules,
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Intended Audience :: Financial and Insurance Industry",
        "License :: OSI Approved :: BSD License",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: Implementation :: PyPy",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
        "Topic :: Office/Business",
        "Topic :: Office/Business :: Financial",
        "Topic :: Office/Business :: Financial :: Accounting",
        "Topic :: Office/Business :: Financial :: Investment",
        "Topic :: Software Development :: Libraries",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    zip_safe=True,
    provides=["setup_template_cython"]
)

print(
    "\n%s v%s installation finished succesfully." % (
        install.get_name().capitalize(),
        install.get_version()
    )
)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""Tests for stats() method"""

from pymarketcap.tests import type_test
from pymarketcap import Pymarketcap
pym = Pymarketcap()

def assert_types(res, convert=None):
    map_types = {
        "active_cryptocurrencies":          int,
        "active_markets":                   int,
        "bitcoin_percentage_of_market_cap": float,
        "quotes":                           dict,
        "last_updated":                     int,
    }
    quotes_map_types = {
        "total_market_cap": float,
        "total_volume_24h": float
    }

    assert isinstance(res, dict)
    for key, value in res.items():
        type_test(map_types, key, value)

    for key, value in res["quotes"]["USD"].items():
        type_test(quotes_map_types, key, value)

    if convert:
        for key, value in res["quotes"][convert].items():
            type_test(quotes_map_types, key, value)

def test_types():
    res = pym.stats()["data"]
    assert_types(res)

def assert_consistence(res):
    assert len(res) == 5

def test_consistence():
    res = pym.stats()["data"]
    assert_consistence(res)

def test_convert():
    fiat_curr = "EUR"
    res = pym.stats(convert=fiat_curr)["data"]
    assert_types(res, convert=fiat_curr)
    assert_consistence(res)



# -*- coding: utf-8 -*-

from pymarketcap import Pymarketcap
from pymarketcap.tests import assert_cryptocurrency_or_exchange_types

pym = Pymarketcap()

NAME = "Poloniex"
WEBSITE_SLUG = "poloniex"


def test_valid_exchange():
    res = pym.exchange_by_field_value("website_slug",
                                      WEBSITE_SLUG)
    assert res["name"] == NAME
    assert res["website_slug"] == WEBSITE_SLUG

    assert_cryptocurrency_or_exchange_types(res)


INVALID_WEBSITE_SLUG = "jishds92327b ugh9 hoGUK"


def test_invalid_exchange():
    res = pym.exchange_by_field_value("website_slug",
                                      INVALID_WEBSITE_SLUG)
    assert res is None



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""Example showing how to get all cryptocurrency
USD prices with Coinmarketcap ticker endpoint pagination."""

from pymarketcap import Pymarketcap
pym = Pymarketcap()

def main():
    start = 1
    num_cryptocurrencies = None
    response = {}

    while True:
        cryptocurrencies = pym.ticker(start=start)
        if not num_cryptocurrencies:
            num_cryptocurrencies = \
                cryptocurrencies["metadata"]["num_cryptocurrencies"]
        for _id, currency in cryptocurrencies["data"].items():
            symbol = cryptocurrencies["data"][_id]["symbol"]
            usd_price = cryptocurrencies["data"][_id]["quotes"]["USD"]["price"]
            if usd_price:
                response[symbol] = usd_price
        start += 100
        if start > num_cryptocurrencies:
            break

    assert len(response) <= num_cryptocurrencies
    return response

if __name__ == '__main__':
    response = main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from pymarketcap import Pymarketcap

pym = Pymarketcap()

tokens = pym.tokens()


def validate(value, may_be_empty=False):
    found = False
    if isinstance(value, str):
        found = may_be_empty or value != ""
    elif isinstance(value, (int, float)):
        found = True
    elif value is None:
        if may_be_empty:
            found = True
    else:
        raise NotImplementedError(
            "Case not anticipated. 'type(value) == %r'" % type(value)
        )
    return found


def assert_any_field_in_response(field, readable_field, may_be_empty=False):
    msg = ("Any %s found searching all currencies. "
           "Check 'processer:tokens()' function." % readable_field)
    try:
        assert all(validate(t[field], may_be_empty) for t in tokens), msg
    except AssertionError as err:
        print(msg)
        raise err


def test_name():
    assert_any_field_in_response("name", "currency name")


def test_symbol():
    assert_any_field_in_response("symbol", "currency symbol")


def test_platform():
    assert_any_field_in_response("platform", "platform")


def test_circulating_supply():
    assert_any_field_in_response(
        "circulating_supply",
        "circulating_supply",
        may_be_empty=True
    )


def test_market_cap():
    assert_any_field_in_response(
        field="market_cap",
        readable_field="market_cap",
        may_be_empty=True
    )


def test_price():
    assert_any_field_in_response("price", "price", may_be_empty=True)


def test_volume_24h():
    assert_any_field_in_response(
        "volume_24h",
        "volume_24h",
        may_be_empty=True
    )



# -*- coding: utf-8 -*-

from random import choice
from pymarketcap.tests import restart_if_http_error

def random_cryptocurrency(instance=None):
    if not instance:
        from pymarketcap import Pymarketcap
        return choice(Pymarketcap().cryptocurrencies)
    else:
        return choice(instance.cryptocurrencies)

def random_exchange(instance=None):
    if not instance:
        from pymarketcap import Pymarketcap
        return choice(Pymarketcap().cryptoexchanges)
    else:
        return choice(instance.cryptoexchanges)



# -*- coding: utf-8 -*-

import json
import time

from pymarketcap import Pymarketcap
from pymarketcap.errors import CoinmarketcapHTTPError

def test_proxy():
    HTTPerror = True
    attempts = 10

    while HTTPerror and attempts > 0:
        # First, check exposed IP without proxy
        try:
            pym_no_proxy = Pymarketcap()
        
            ip_url = b"http://httpbin.org/ip"
        
            response = pym_no_proxy._get(ip_url)
            original_ip = json.loads(response)["origin"]
        
            # Second, check exposed IP using Tor proxy
            # You need to install Tor
            pym_proxy = Pymarketcap(proxy_addr=b"socks5://127.0.0.1:9050")
            response = pym_proxy._get(ip_url)
        
            masked_ip = json.loads(response)["origin"]
        except CoinmarketcapHTTPError as err:
        	attempts -= 1
        	time.sleep(2)
        else:
        	HTTPerror = False

    if not HTTPerror:
        assert original_ip != masked_ip
        return
    print("Max attempts reached retrieving IP from: %s" % str(ip_url))
    raise err





#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import os
from time import sleep
from random import choice
from urllib.error import HTTPError

import pytest
from tqdm import tqdm

from pymarketcap.tests.utils import random_cryptocurrency
from pymarketcap import Pymarketcap
pym = Pymarketcap()

def test_consistence():
    print("")
    for size in tqdm([16, 32, 64, 128, 200],
                     desc="Testing currency logo downloads for all sizes"):
        attempts = 20
        while attempts > 0:
            _assert = True
            website_slug = random_cryptocurrency(pym)["website_slug"]
            tqdm.write(('<currency>["website_slug"] == "%s" | Size: %d)' \
                % (website_slug, size))
            )
            try:
                res = pym.download_logo(website_slug, size=size)
            except ValueError as e:
                print(e)
                attempts -= 1
                _assert = False
                if attempts == 0:
                    break
            else:
                break
        if _assert:
            assert res == "%s_%dx%d.png" % (website_slug, size, size)
            sleep(.5)

            assert os.path.exists(res)
            os.remove(res)
            assert os.path.exists(res) == False
        if attempts == 0:
            raise AssertionError("0 currency logos downloaded. Check it!")

def test_invalid():
    # Invalid currencies
    symbol = "OADVDOVASDYIV"
    with pytest.raises(ValueError) as excinfo:
        pym.download_logo(symbol)
    expected_msg = "Any cryptocurrency found matching symbol == %r." % symbol
    assert expected_msg in str(excinfo)

    # Invalid size
    size = 250
    with pytest.raises(ValueError) as excinfo:
        pym.download_logo("BTC", size=size)
    assert "%dx%d is not a valid size." % (size, size) in str(excinfo)

    # Valid size that doesn't exist for a currency.
    symbol = "GMX"
    size = 200
    with pytest.raises(ValueError) as excinfo:
        pym.download_logo(symbol, size=size)
    assert "currency doesn't allows to be downloaded with size %dx%d." \
        % (size, size) in str(excinfo)
